"""
Class Factory for NDG SAML package

NERC DataGrid project
"""
__author__ = "Philip Kershaw"
__date__ = "15/02/10"
__copyright__ = "(C) 2010 Science and Technology Facilities Council"
__license__ = "http://www.apache.org/licenses/LICENSE-2.0"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = '$Id:$'
import traceback
import logging
import os
import sys
import re
from ConfigParser import ConfigParser, SafeConfigParser
from abc import ABCMeta, abstractmethod

from ndg.saml.saml2 import core as saml2

log = logging.getLogger(__name__)


def importModuleObject(moduleName, objectName=None, objectType=None):
    '''Import from a string module name and object name.  Object can be
    any entity contained in a module
    
    @param moduleName: Name of module containing the class
    @type moduleName: str 
    @param objectName: Name of the class to import.  If none is given, the 
    class name will be assumed to be the last component of modulePath
    @type objectName: str
    @rtype: class object
    @return: imported class'''
    if objectName is None:
        if ':' in moduleName:
            # Support Paste style import syntax with rhs of colon denoting 
            # module content to import
            _moduleName, objectName = moduleName.rsplit(':', 1)
            if '.' in objectName:
                objectName = objectName.split('.')
        else: 
            try:
                _moduleName, objectName = moduleName.rsplit('.', 1)
            except ValueError:
                raise ValueError('Invalid module name %r set for import: %s' %
                                 (moduleName, traceback.format_exc()))
    else:
        _moduleName = moduleName
        
    if isinstance(objectName, basestring):
        objectName = [objectName]
    
    log.debug("Importing %r ..." % objectName) 
      
    module = __import__(_moduleName, globals(), locals(), [])
    components = _moduleName.split('.')
    try:
        for component in components[1:]:
            module = getattr(module, component)
    except AttributeError:
        raise AttributeError("Error importing %r: %s" %
                             (objectName, traceback.format_exc()))

    importedObject = module
    for i in objectName:
        importedObject = getattr(importedObject, i)

    # Check class inherits from a base class
    if objectType and not issubclass(importedObject, objectType):
        raise TypeError("Specified class %r must be derived from %r; got %r" %
                        (objectName, objectType, importedObject))
    
    log.info('Imported %r from module, %r', objectName, _moduleName)
    return importedObject


def callModuleObject(moduleName, objectName=None, moduleFilePath=None, 
                     objectType=None, objectArgs=None, objectProperties=None):
    '''
    Create and return an instance of the specified class or invoke callable
    @param moduleName: Name of module containing the class
    @type moduleName: str 
    @param objectName: Name of the class to instantiate.  May be None in 
    which case, the class name is parsed from the moduleName last element
    @type objectName: str
    @param moduleFilePath: Path to the module - if unset, assume module on 
    system path already
    @type moduleFilePath: str
    @param objectProperties: dict of properties to use when instantiating the 
    class
    @type objectProperties: dict
    @param objectType: expected type for the object to instantiate - to 
    enforce use of specific interfaces 
    @type objectType: object
    @return: object - instance of the class specified 
    '''

    # ensure that properties is a dict - NB, it may be passed in as a null
    # value which can override the default val
    if not objectProperties:
        objectProperties = {}

    if not objectArgs:
        objectArgs = ()
        
    # variable to store original state of the system path
    sysPathBak = None
    try:
        try:
            # Module file path may be None if the new module to be loaded
            # can be found in the existing system path            
            if moduleFilePath:
                if not os.path.exists(moduleFilePath):
                    raise IOError("Module file path '%s' doesn't exist" % 
                                  moduleFilePath)
                          
                # Temporarily extend system path ready for import
                sysPathBak = sys.path
                          
                sys.path.append(moduleFilePath)

            
            # Import module name specified in properties file
            importedObject = importModuleObject(moduleName, 
                                                objectName=objectName,
                                                objectType=objectType)
        finally:
            # revert back to original sys path, if necessary
            # NB, python requires the use of a try/finally OR a try/except 
            # block - not both combined
            if sysPathBak:
                sys.path = sysPathBak
                            
    except Exception, e:
        log.error('%r module import raised %r type exception: %r' % 
                  (moduleName, e.__class__, traceback.format_exc()))
        raise 

    # Instantiate class
    log.debug('Instantiating object "%s"' % importedObject.__name__)
    try:
        if objectArgs:
            obj = importedObject(*objectArgs, **objectProperties)
        else:
            obj = importedObject(**objectProperties)
            
        return obj

    except Exception, e:
        log.error("Instantiating module object, %r: %r" % 
                                                    (importedObject.__name__, 
                                                     traceback.format_exc()))
        raise
    

class SubjectFactory(object):
    '''Factory class to create Subject instance'''
    
    @classmethod
    def create(cls):
        '''Create a blank subject with name id instantiated
        '''
        subject = saml2.Subject()
        subject.nameID = saml2.NameID()
        
        return subject


class QueryFactoryBase(object):
    """Abstract base Factory class to create SAML queries from various 
    inputs - derived classes determine the query types
    """
    __metaclass__ = ABCMeta
    
    QUERY_CLASS = None
    PREFIX = None
    SUBJECT_PARAM_NAME_PREFIX = 'subject.'
    ISSUER_PARAM_NAME_PREFIX = 'issuer.'
    ATTR_PARAM_VAL_SEP_PAT = re.compile(',\s*')
    ATTR_PARAM_NAME_PREFIX = 'attributes.'
    
    @classmethod
    def create(cls):
        '''Create a blank attribute query with all member variables instantiated
         - issuer, subject etc.
        '''
        if cls.QUERY_CLASS is None:
            raise NotImplementedError('"QUERY_CLASS" class variable must be '
                                      'set in derived class')
            
        query = cls.QUERY_CLASS()
        query.version = saml2.SAMLVersion(saml2.SAMLVersion.VERSION_20)
        query.subject = SubjectFactory.create()
        query.issuer = saml2.Issuer()
        
        # Default to X.509 subject name for format
        query.issuer.format = saml2.Issuer.X509_SUBJECT
        
        return query
   
    @classmethod
    def from_config(cls, cfg, prefix=PREFIX, section='DEFAULT'):
        '''Create query from config file settings
        @type cfg: basestring /ConfigParser derived type
        @param cfg: configuration file path or ConfigParser type object
        @type prefix: basestring
        @param prefix: prefix for option names e.g. "attributeQuery."
        @type section: baestring
        @param section: configuration file section from which to extract
        parameters.
        '''  
        if isinstance(cfg, basestring):
            cfg_filepath = os.path.expandvars(cfg)
            here_dir = os.path.dirname(cfg_filepath)
            _cfg = SafeConfigParser(defaults=dict(here=here_dir))
            _cfg.optionxform = str

            _cfg.read(cfg_filepath)
            
        elif isinstance(cfg, ConfigParser):
            _cfg = cfg   
        else:
            raise AttributeError('Expecting basestring or ConfigParser type '
                                 'for "cfg" attribute; got %r type' % type(cfg))
        
        # Get items for this section as a dictionary so that from_kw can
        # used to update the object
        kw = dict(_cfg.items(section))
        if 'prefix' not in kw and prefix:
            kw['prefix'] = prefix
            
        return cls.from_kw(**kw)
    
    @classmethod
    @abstractmethod
    def from_kw(cls, prefix=PREFIX, **config):
        '''parse attribute query from an input keywords'''
           
    
class AttributeQueryFactory(QueryFactoryBase):
    """Factory class to create attribute queries from various inputs
    """
    PREFIX = 'attribute_query.'
    QUERY_CLASS = saml2.AttributeQuery
    SUBJECT_PARAM_NAME_PREFIX = 'subject.'
    ISSUER_PARAM_NAME_PREFIX = 'issuer.'
    ATTR_PARAM_VAL_SEP_PAT = re.compile(',\s*')
    ATTR_PARAM_NAME_PREFIX = 'attributes.'

    @classmethod
    def from_kw(cls, prefix=PREFIX, **config):
        '''parse attribute query from an input keywords'''
        attribute_query = cls.create()
        pat = cls.ATTR_PARAM_VAL_SEP_PAT
        
        for param_name, param_val in config.items():
            
            # Skip values that don't start with the correct prefix
            if not param_name.startswith(prefix):
                continue
            
            _param_name = param_name.rsplit(prefix, 1)[-1]
            
            # Check for items which have the same name as AttributeQuery
            # object member variables
            if _param_name.startswith(cls.SUBJECT_PARAM_NAME_PREFIX):
                nameid_param_name = _param_name.rsplit('subject.nameID.')[-1]
                
                setattr(attribute_query.subject.nameID, nameid_param_name, 
                        param_val)
                
            elif _param_name.startswith(cls.ISSUER_PARAM_NAME_PREFIX):
                issuer_param_name = _param_name.rsplit('issuer.')[-1]
                setattr(attribute_query.issuer, issuer_param_name, 
                        param_val)
                
            elif _param_name.startswith(cls.ATTR_PARAM_NAME_PREFIX):
                # attributes are set with a special syntax.  Each attribute
                # name in the dictionary should start with 
                # ``prefix`` and end with some unique string
                attribute = saml2.Attribute()
                
                # The values should be parsed from a string containing a 
                # comma-separated list e.g.
                #
                # attribute.0 = urn:esg:first:name, FirstName, http://www.w3.org/2001/XMLSchema#string
                (attribute.name, 
                 attribute.friendlyName, 
                 attribute.nameFormat) = pat.split(param_val)
         
                attribute_query.attributes.append(attribute)
            else:
                raise AttributeError('Config item %r not recognised as a valid '
                                     'AttributeQuery object member variable.' %(
                                                                 param_name))
            
        return attribute_query


class AuthzDecisionQueryFactory(QueryFactoryBase):
    """Factory class to create authorisation decision queries from various 
    inputs
    """
    PREFIX = 'authz_decision_query.'
    QUERY_CLASS = saml2.AuthzDecisionQuery
    SUBJECT_PARAM_NAME_PREFIX = 'subject.'
    ISSUER_PARAM_NAME_PREFIX = 'issuer.'
    ATTR_PARAM_VAL_SEP_PAT = re.compile(',\s*')
    ATTR_PARAM_NAME_PREFIX = 'attributes.'
    
    @classmethod
    def from_kw(cls, prefix=PREFIX, **config):
        '''parse attribute query from an input keywords'''
        authz_decision_query = cls.create()
        pat = cls.ATTR_PARAM_VAL_SEP_PAT
        
        for param_name, param_val in config.items():
            
            # Skip values that don't start with the correct prefix
            if not param_name.startswith(prefix):
                continue
            
            _param_name = param_name.rsplit(prefix, 1)[-1]
            
            # Check for items which have the same name as AttributeQuery
            # object member variables
            if _param_name.startswith(cls.SUBJECT_PARAM_NAME_PREFIX):
                nameid_param_name = _param_name.rsplit('subject.nameID.')[-1]
                
                setattr(authz_decision_query.subject.nameID, nameid_param_name, 
                        param_val)
                
            elif _param_name.startswith(cls.ISSUER_PARAM_NAME_PREFIX):
                issuer_param_name = _param_name.rsplit('issuer.')[-1]
                setattr(authz_decision_query.issuer, issuer_param_name, 
                        param_val)
                
            elif _param_name.startswith(cls.ATTR_PARAM_NAME_PREFIX):
                # attributes are set with a special syntax.  Each attribute
                # name in the dictionary should start with 
                # ``prefix`` and end with some unique string
                attribute = saml2.Attribute()
                
                # The values should be parsed from a string containing a 
                # comma-separated list e.g.
                #
                # attribute.0 = urn:esg:first:name, FirstName, http://www.w3.org/2001/XMLSchema#string
                (attribute.name, 
                 attribute.friendlyName, 
                 attribute.nameFormat) = pat.split(param_val)
         
                authz_decision_query.attributes.append(attribute)
            else:
                raise AttributeError('Config item %r not recognised as a valid '
                                     'AuthzDecisionbQuery object member '
                                     'variable.' % param_name)
            
        return authz_decision_query


'''SAML factory module unit test package


This implementation is adapted from the Java OpenSAML implementation.  The 
copyright and licence information are included here:

Copyright [2005] [University Corporation for Advanced Internet Development, Inc.]

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
'''
__author__ = "P J Kershaw"
__date__ = "16/07/15"
__copyright__ = "(C) 2015 Science and Technology Facilities Council"
__license__ = "http://www.apache.org/licenses/LICENSE-2.0"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = '$Id$'
import unittest

from ndg.saml.utils.factory import (AttributeQueryFactory, 
                                    AuthzDecisionQueryFactory)


class AttributeQueryFactoryTestCase(unittest.TestCase):
    '''Test attribute query factory class'''
    def setUp(self):
        self.config = {
            'attributeQuery.subject.nameID.format': 'urn:esg:openid',
            'attributeQuery.issuer.value': '/O=Site A/CN=Authorisation Service',
            'attributeQuery.attributes.0': 
    'urn:esg:first:name, FirstName, http://www.w3.org/2001/XMLSchema#string',
            'attributeQuery.attributes.roles': 
'urn:siteA:security:authz:1.0:attr, , http://www.w3.org/2001/XMLSchema#string'  
        }
        
    def test01_create(self):
        attribute_query = AttributeQueryFactory.create()
        self.assertIsNotNone(attribute_query.subject, 'query subject is none')
        self.assertIsNotNone(attribute_query.issuer, 'query issuer is none')
        
    def test02_from_kw(self):
        attribute_query = AttributeQueryFactory.from_kw(
                                                    prefix='attributeQuery.',
                                                    **self.config)
        
        self.assertEqual(attribute_query.subject.nameID.format, 
                         self.config['attributeQuery.subject.nameID.format'], 
                         'Parameter is %r, expected %r' % (
                          attribute_query.subject.nameID.format, 
                          self.config['attributeQuery.subject.nameID.format']))
        
        self.assertEqual(attribute_query.issuer.value, 
                         self.config['attributeQuery.issuer.value'], 
                         'Parameter is %r, expected %r' % (
                         attribute_query.issuer.value, 
                         self.config['attributeQuery.issuer.value']))
        
        self.assertEqual(len(attribute_query.attributes), 2, 
                        'expecting 2 SAML attributes parsed')
        
        attr = None
        for attr in attribute_query.attributes:
            if attr.friendlyName == 'FirstName':
                break
            
        self.assertNotEqual(attr, None, 
                            'Missing expected friendlyName attribute')
                           
        self.assertIn(attr.nameFormat, 
                      self.config['attributeQuery.attributes.0'], 
                      'Parameter is %r, not found in %r' % (
                          attr.nameFormat, 
                          self.config['attributeQuery.attributes.0']))

        self.assertIn(attr.name, 
                      self.config['attributeQuery.attributes.0'], 
                      'Parameter is %r, not found in %r' % (
                          attr.name, 
                          self.config['attributeQuery.attributes.0']))


class AuthzDecisionQueryFactoryTestCase(unittest.TestCase):
    '''Test authorisation decision query factory class'''
    def setUp(self):
        self.config = {
            'authz_q.subject.nameID.format': 'urn:esg:openid',
            'authz_q.issuer.value': '/O=Site A/CN=Authorisation Service',
        }
        
    def test01_create(self):
        authz_query = AuthzDecisionQueryFactory.create()
        self.assertIsNotNone(authz_query.subject, 'query subject is none')
        self.assertIsNotNone(authz_query.issuer, 'query issuer is none')
        
    def test02_from_kw(self):
        authz_query = AuthzDecisionQueryFactory.from_kw(
                                                    prefix='authz_q.',
                                                    **self.config)
        
        self.assertEqual(authz_query.subject.nameID.format, 
                         self.config['authz_q.subject.nameID.format'], 
                         'Parameter is %r, expected %r' % (
                          authz_query.subject.nameID.format, 
                          self.config['authz_q.subject.nameID.format']))
        
        self.assertEqual(authz_query.issuer.value, 
                         self.config['authz_q.issuer.value'], 
                         'Parameter is %r, expected %r' % (
                         authz_query.issuer.value, 
                         self.config['authz_q.issuer.value']))
        
        
if __name__ == "__main__":
    unittest.main()


"""SAML 2.0 bindings module implements SOAP binding for base request

NERC DataGrid Project
"""
__author__ = "P J Kershaw"
__date__ = "12/02/10"
__copyright__ = "(C) 2010 Science and Technology Facilities Council"
__license__ = "http://www.apache.org/licenses/LICENSE-2.0"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = '$Id: subjectquery.py 7634 2010-10-20 20:23:29Z pjkersha $'
import logging
log = logging.getLogger(__name__)

from datetime import datetime, timedelta
from uuid import uuid4

from ndg.saml.utils import SAMLDateTime
from ndg.saml.saml2.core import RequestAbstractType, StatusCode

from ndg.saml.utils import str2Bool
from ndg.saml.saml2.binding.soap.client import (SOAPBinding,
                                                SOAPBindingInvalidResponse)


class RequestResponseError(SOAPBindingInvalidResponse):
    """SAML Response error from request"""
    

class IssueInstantInvalid(RequestResponseError):
    """Issue instant of SAML artifact is invalid"""

  
class ResponseIssueInstantInvalid(IssueInstantInvalid):
    """Issue instant of a response is after the current time"""

    
class AssertionIssueInstantInvalid(IssueInstantInvalid):
    """Issue instant of an assertion is after the current time"""


class AssertionConditionNotBeforeInvalid(RequestResponseError):
    """An assertion condition notBefore time is set after the current clock
    time"""
    

class AssertionConditionNotOnOrAfterInvalid(RequestResponseError):
    """An assertion condition notOnOrAfter time is set before the current clock
    time"""

   
class RequestBaseSOAPBinding(SOAPBinding): 
    """SAML Request Base SOAP Binding
    """
    CLOCK_SKEW_OPTNAME = 'clockSkewTolerance'
    VERIFY_TIME_CONDITIONS_OPTNAME = 'verifyTimeConditions'
    
    CONFIG_FILE_OPTNAMES = (
        CLOCK_SKEW_OPTNAME,
        VERIFY_TIME_CONDITIONS_OPTNAME            
    )
    
    __PRIVATE_ATTR_PREFIX = "__"
    __slots__ = tuple([__PRIVATE_ATTR_PREFIX + i 
                       for i in CONFIG_FILE_OPTNAMES + ('issuer',)])
    del i
    
    QUERY_TYPE = RequestAbstractType
    
    def __init__(self, **kw):
        '''Create SOAP Client for a SAML Subject Query'''       
        self.__clockSkewTolerance = timedelta(seconds=0.)
        self.__verifyTimeConditions = True
        
        super(RequestBaseSOAPBinding, self).__init__(**kw)

    def _getVerifyTimeConditions(self):
        return self.__verifyTimeConditions

    def _setVerifyTimeConditions(self, value):
        if isinstance(value, bool):
            self.__verifyTimeConditions = value
            
        if isinstance(value, basestring):
            self.__verifyTimeConditions = str2Bool(value)
        else:
            raise TypeError('Expecting bool or string type for '
                            '"verifyTimeConditions"; got %r instead' % 
                            type(value))

    verifyTimeConditions = property(_getVerifyTimeConditions, 
                                    _setVerifyTimeConditions, 
                                    doc='Set to True to verify any time '
                                        'Conditions set in the returned '
                                        'response assertions')  

    def _getClockSkewTolerance(self):
        return self.__clockSkewTolerance

    def _setClockSkewTolerance(self, value):
        if isinstance(value, timedelta):
            self.__clockSkewTolerance = value
            
        elif isinstance(value, (float, int, long)):
            self.__clockSkewTolerance = timedelta(seconds=value)
            
        elif isinstance(value, basestring):
            self.__clockSkewTolerance = timedelta(seconds=float(value))
        else:
            raise TypeError('Expecting timedelta, float, int, long or string '
                            'type for "clockSkewTolerance"; got %r' % 
                            type(value))

    clockSkewTolerance = property(fget=_getClockSkewTolerance, 
                                  fset=_setClockSkewTolerance, 
                                  doc="Allow a tolerance in seconds for SAML "
                                      "Query issueInstant parameter check and "
                                      "assertion condition notBefore and "
                                      "notOnOrAfter times to allow for clock "
                                      "skew")
    
    def _validateQueryParameters(self, query):
        """Perform sanity check immediately before creating the query and 
        sending it"""
        errors = []
        
        if query.issuer is None or query.issuer.value is None:
            errors.append('issuer name')

        if query.issuer is None or query.issuer.format is None:
            errors.append('issuer format')
        
        if errors:
            raise AttributeError('Missing attribute(s) for SAML Query: %s' %
                                 ', '.join(errors))

    def _initSend(self, query):
        """Perform any final initialisation prior to sending the query - derived
        classes may overload to specify as required"""
        query.issueInstant = datetime.utcnow()
        
        # Set ID here to ensure it's unique for each new call
        query.id = str(uuid4())

    def _verifyTimeConditions(self, response):
        """Verify time conditions set in a response
        :param response: SAML Response returned from remote service
        :type response: ndg.saml.saml2.core.Response
        :raise RequestResponseError: if a timestamp is invalid
        """
        
        if not self.verifyTimeConditions:
            log.debug("Skipping verification of SAML Response time conditions")
            
        utcNow = datetime.utcnow() 
        nowMinusSkew = utcNow - self.clockSkewTolerance
        nowPlusSkew = utcNow + self.clockSkewTolerance
        
        if response.issueInstant > nowPlusSkew:
            msg = ('SAML Attribute Response issueInstant [%s] is after '
                   'the clock time [%s] (skewed +%s)' % 
                   (response.issueInstant, 
                    SAMLDateTime.toString(nowPlusSkew),
                    self.clockSkewTolerance))
             
            samlRespError = ResponseIssueInstantInvalid(msg)
            samlRespError.response = response
            raise samlRespError
        
        for assertion in response.assertions:
            if assertion.issueInstant is None:
                samlRespError = AssertionIssueInstantInvalid("No issueInstant "
                                                             "set in response "
                                                             "assertion")
                samlRespError.response = response
                raise samlRespError
            
            elif nowPlusSkew < assertion.issueInstant:
                msg = ('The clock time [%s] (skewed +%s) is before the '
                       'SAML Attribute Response assertion issue instant [%s]' % 
                       (SAMLDateTime.toString(utcNow),
                        self.clockSkewTolerance,
                        assertion.issueInstant))
                samlRespError = AssertionIssueInstantInvalid(msg)
                samlRespError.response = response
                raise samlRespError
            
            if assertion.conditions is not None:
                if nowPlusSkew < assertion.conditions.notBefore:            
                    msg = ('The clock time [%s] (skewed +%s) is before the '
                           'SAML Attribute Response assertion conditions not '
                           'before time [%s]' % 
                           (SAMLDateTime.toString(utcNow),
                            self.clockSkewTolerance,
                            assertion.conditions.notBefore))
                              
                    samlRespError = AssertionConditionNotBeforeInvalid(msg)
                    samlRespError.response = response
                    raise samlRespError
                 
                if nowMinusSkew >= assertion.conditions.notOnOrAfter:           
                    msg = ('The clock time [%s] (skewed -%s) is on or after '
                           'the SAML Attribute Response assertion conditions '
                           'not on or after time [%s]' % 
                           (SAMLDateTime.toString(utcNow),
                            self.clockSkewTolerance,
                            assertion.conditions.notOnOrAfter))
                    
                    samlRespError = AssertionConditionNotOnOrAfterInvalid(msg) 
                    samlRespError.response = response
                    raise samlRespError
                
    def send(self, query, **kw):
        '''Make an attribute query to a remote SAML service
        
        :type uri: basestring 
        :param uri: uri of service.  May be omitted if set from request.url
        :type request: ndg.security.common.soap.UrlLib2SOAPRequest
        :param request: SOAP request object to which query will be attached
        defaults to ndg.security.common.soap.client.UrlLib2SOAPRequest
        '''
        self._validateQueryParameters(query)
        self._initSend(query)
           
        log.debug("Sending request: query ID: %s", query.id)
        response = super(RequestBaseSOAPBinding, self).send(query, **kw)

        # Perform validation - Nb. status message may be None
        if response.status.statusCode.value != StatusCode.SUCCESS_URI:
            # Allow for server response missing status message
            status_msg = getattr(response.status, 'statusMessage')
            if not status_msg:
                status_msg_val = ''
            else:
                status_msg_val = status_msg.value
                
            msg = ('Return status code flagged an error, %r.  '
                   'The message is, %r' %
                   (response.status.statusCode.value, status_msg_val)) 
            samlRespError = RequestResponseError(msg)
            samlRespError.response = response
            raise samlRespError
        
        # Check Query ID matches the query ID the service received
        if response.inResponseTo != query.id:
            msg = ('Response in-response-to ID %r, doesn\'t match the original '
                   'query ID, %r' % (response.inResponseTo, query.id))
            
            samlRespError = RequestResponseError(msg)
            samlRespError.response = response
            raise samlRespError
        
        self._verifyTimeConditions(response)
            
        return response 



"""SAML SOAP Binding Query/Response Interface with service hosted in
Paste paster web server

NERC DataGrid Project
"""
__author__ = "P J Kershaw"
__date__ = "01/07/10"
__copyright__ = "(C) 2010 Science and Technology Facilities Council"
__license__ = "http://www.apache.org/licenses/LICENSE-2.0"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = '$Id$'
import logging
logging.basicConfig(level=logging.DEBUG)

import unittest
from os import path
from ndg.saml import importElementTree
ElementTree = importElementTree()

from ndg.soap.utils.etree import prettyPrint

from ndg.saml.saml2.core import Attribute, StatusCode
from ndg.saml.xml.etree import ResponseElementTree
from ndg.saml.saml2.binding.soap.client.attributequery import \
    AttributeQuerySslSOAPBinding
from ndg.saml.utils.factory import AttributeQueryFactory
from ndg.saml.test.binding.soap import WithPasterBaseTestCase, paste_installed
    
    
class SamlSslSoapBindingTestCase(WithPasterBaseTestCase):
    """Test SAML SOAP Binding with SSL"""
    SERVICE_URI = 'https://localhost:5443/attributeauthority'
    SUBJECT = "https://openid.localhost/philip.kershaw"
    SUBJECT_FORMAT = "urn:ndg:saml:openid"
    CONFIG_FILENAME = 'attribute-interface.ini'
    
    CLIENT_CERT_FILEPATH = path.join(WithPasterBaseTestCase.THIS_DIR, 
                                     'localhost.crt')
    CLIENT_PRIKEY_FILEPATH = path.join(WithPasterBaseTestCase.THIS_DIR, 
                                       'localhost.key')
    CLIENT_CACERT_DIR = path.join(WithPasterBaseTestCase.THIS_DIR, 'ca')
    VALID_DNS = [
        '/O=NDG/OU=Security/CN=localhost', 
    ]
    
    @unittest.skipIf(not paste_installed, 'Need Paste.Deploy to run '
                     'SamlSslSoapBindingTestCase')
    
    def __init__(self, *arg, **kw):
        kw['withSSL'] = True
        super(SamlSslSoapBindingTestCase, self).__init__(*arg, **kw)
                    
    def test02SendQuery(self):
        query_binding = AttributeQuerySslSOAPBinding()
        
        attribute_query = AttributeQueryFactory.create()
        attribute_query.subject.nameID.format = self.__class__.SUBJECT_FORMAT
        attribute_query.subject.nameID.value = self.__class__.SUBJECT
        attribute_query.issuerName = '/O=Site A/CN=Authorisation Service'


        attribute = Attribute()
        attribute.name = 'urn:ndg:saml:emailaddress'
        attribute.friendlyName = 'emailAddress'
        attribute.nameFormat = 'http://www.w3.org/2001/XMLSchema'
        
        attribute_query.attributes.append(attribute)
        
        query_binding.clockSkewTolerance = 2.
        query_binding.sslCACertDir = self.__class__.CLIENT_CACERT_DIR
        query_binding.sslCertFilePath = self.__class__.CLIENT_CERT_FILEPATH
        query_binding.sslPriKeyFilePath = self.__class__.CLIENT_PRIKEY_FILEPATH
        query_binding.sslValidDNs = self.__class__.VALID_DNS
        
        response = query_binding.send(attribute_query, 
                                      uri=self.__class__.SERVICE_URI)
        
        # Convert back to ElementTree instance read for string output
        samlResponseElem = ResponseElementTree.toXML(response)
        
        print("SAML Response ...")
        print(ElementTree.tostring(samlResponseElem))
        print("Pretty print SAML Response ...")
        print(prettyPrint(samlResponseElem))
        
        self.assert_(response.status.statusCode.value==StatusCode.SUCCESS_URI)
  
 
if __name__ == "__main__":
    if paste_installed:
        unittest.main()
    else:
        import warnings
        warnings.warn('Skip unittests for %r, Paste package is not installed' %
                      __name__)



'''
Created on Feb 18, 2013

@author: philipkershaw
'''
import sys
from optparse import OptionParser
from uuid import uuid4
from datetime import datetime

from ndg.saml import importElementTree
ElementTree = importElementTree()

from ndg.soap.utils.etree import prettyPrint

from ndg.saml.saml2.binding.soap.client.authzdecisionquery import \
    AuthzDecisionQuerySslSOAPBinding
    
from ndg.saml.saml2.binding.soap.client.attributequery import \
    AttributeQuerySslSOAPBinding
    
from ndg.saml.saml2.core import (SAMLVersion, Subject, NameID, Issuer, 
                                 AuthzDecisionQuery, AttributeQuery,
                                 Attribute, Action, StatusCode,
                                 XSStringAttributeValue)
from ndg.saml.xml.etree import ResponseElementTree


class SamlSoapCommandLineClient(object):
    '''Simple SAML SOAP Client'''
    CONFIG_FILENAME = 'authz-decision-interface.ini'
    
    __slots__ = (
        "service_uri",
        "issuer", 
        "issuer_format",
        "subject_id", 
        "subject_id_format",
        "resource_id", 
        "action", 
        "action_namespace",
        "attribute_names",
        "attribute_friendly_names",
        "attribute_formats",
        "ca_cert_dir",
        "client_cert_filepath",
        "client_prikey_filepath",
        "pretty_print",
        "clock_skew_tolerance",
        "debug"
    )
    
    ATTRIBUTE_QUERY_CMD = 'attr'
    AUTHZ_DECISION_QUERY_CMD = 'authz'
    
    def __init__(self):
        for i in self.__class__.__slots__:
            setattr(self, i, None)
        
    def parse_command_line(self, argv):
        usage = """usage: %prog [command] [options]

commands:
  authz       Make a SAML Authorisation Decision Query
  attr        Make a SAML Attribute Query
"""
        parser = OptionParser(usage=usage)
        
        # List of generic options
        parser.add_option("-u", "--service_uri", dest="service_uri",
                          help="URI for SAML service", 
                          default='',
                          metavar="SERVICE_URI")
        
        parser.add_option("-i", "--issuer", dest="issuer",
                          help="Identity of issuer of this request", 
                          default="/O=Site A/CN=PEP",
                          metavar="ISSUER")
        
        parser.add_option("-I", "--issuer_format", dest="issuer_format",
                          help="Format for request issuer, default is X.509 "
                               "subject name", 
                          default=Issuer.X509_SUBJECT,
                          metavar="ISSUER_FORMAT")
                               
        parser.add_option("-s", "--subject",
                          dest="subject_id", 
                          help="Subject ID about which the request is being "
                               "made - typically a user ID or username",
                          default='',
                          metavar="SUBJECT_ID")
                               
        parser.add_option("-S", "--subject_id_format",
                          dest="subject_id_format", 
                          help="Format for subject ID, defaults to X.509 "
                               "subject name format",
                          default=NameID.X509_SUBJECT,
                          metavar="SUBJECT_ID_FORMAT") 
   
        parser.add_option("-c", "--cert",
                          dest="client_cert_filepath", 
                          help="SSL client certificate for authentication with "
                               "service",
                          metavar="CERT")
   
        parser.add_option("-k", "--key",
                          dest="client_prikey_filepath", 
                          help="SSL client private key for authentication with "
                               "service",
                          metavar="KEY")
   
        parser.add_option("-C", "--ca-cert-dir",
                          dest="ca_cert_dir", 
                          help="Directory containing accepted CA certificates. "
                               "Peer must have an SSL certificate issued from "
                               "one of these.",
                          metavar="CA_CERT_DIR")
   
        parser.add_option("-p", "--pretty-print",
                          dest="pretty_print", 
                          action="store_true",
                          help="Set to true to pretty print SAML responses",
                          metavar="PRETTY_PRINT")
                  
        parser.add_option("-t", "--clock_skew_tolerance",
                          dest="clock_skew_tolerance", 
                          help="Permitted tolerance of +/- n seconds for clock "
                               "skew between client and server when verifying "
                               "time stamps",
                          default=1.,
                          type="float",
                          metavar="CLOCK_SKEW_TOLERANCE")
        
        parser.add_option("-d", "--debug",
                          dest="debug", 
                          action="store_true",
                          help="Set log level to debug for additional error "
                               "output",
                          metavar="DEBUG")
        
        # Allow syntax whereby the first argument is a command - much as openssl
        # does
        n_args = len(argv)
        if n_args < 2:
            parser.error('No command set')
        else:
            command = argv[1]
        
        # Catch example of just specifying --help or '-h'
        if command in ('--help', '-h'):
            parser.print_usage()
            return
          
        elif command == self.__class__.AUTHZ_DECISION_QUERY_CMD:
            # Set options which are specific to authorisation decision queries            
            parser.set_usage('usage: %prog ' + \
                             self.__class__.AUTHZ_DECISION_QUERY_CMD + \
                             ' [options]')
                             
            parser.add_option("-r", "--resource",
                              dest="resource_id", 
                              help="Resource ID to check for access to "
                                   "(for testing authorisation request ONLY)",
                              default='',
                              metavar="RESOURCE_URI")
                                          
            parser.add_option("-a", "--action",
                              dest="action", 
                              help="Requested action - value determined by "
                                   "Action namespace e.g. GET for action "
                                   "namespace %r" % Action.HTTP_GET_ACTION,
                              default=Action.HTTP_GET_ACTION,
                              metavar="ACTION")
                               
            parser.add_option("-A", "--action_namespace",
                              dest="action_namespace", 
                              help="Namespace for requested action",
                              default=Action.GHPP_NS_URI,
                              metavar="ACTION_NS")
                
        elif command == self.__class__.ATTRIBUTE_QUERY_CMD:
            # Set options which are specific to attribute queries            
            parser.set_usage('usage: %prog ' + \
                             self.__class__.ATTRIBUTE_QUERY_CMD + \
                             ' [options]')
            
            parser.add_option("-a", "--attribute_name",
                              dest="attribute_names",
                              action="append", 
                              help="Query for a specific attribute name.  This "
                                   "option can be provided more than once in "
                                   "order to query more than one name.  If "
                                   "this option is not set, a generic query of "
                                   "all available attributes will be made.  "
                                   "This option should be used in conjunction "
                                   "with options the -f/--"
                                   "attribute_friendly_name and "
                                   "-F/--attribute_format.",
                              default=[],
                              metavar="ATTRIBUTE_NAME")
            
            parser.add_option("-f", "--attribute_friendly_name",
                              dest="attribute_friendly_names",
                              action="append", 
                              help="Friendly name is an optional alias to "
                                   "attribute name.  It can be used more than "
                                   "once to specify multiple attributes but "
                                   "if used, make sure it corresponds exactly "
                                   "to the '--attribute_name' settings",
                              default=[],
                              metavar="ATTRIBUTE_FRIENDLY_NAME")            
            
            parser.add_option("-F", "--attribute_format",
                              dest="attribute_formats",
                              action="append", 
                              help="Set the format of the attribute set with "
                                   "the corresponding '--attribute_name'. "
                                   "The option must be set the same number of "
                                   "times as '--attribute_name'.  However, if "
                                   "you wish ALL attributes to use the same "
                                   "format, it is possible to indicate this by "
                                   "setting it just once.",
                              default=[
                                XSStringAttributeValue.TYPE_NAME.namespaceURI],
                              metavar="ATTRIBUTE_FORMAT")
        elif n_args < 3:
            parser.error('No command options set')
            
        else:
            parser.error('Command %s not supported' % command)

        # Leave the command option out of the parser's processing
        options = parser.parse_args(argv[2:])[0]
        
        # Post-processing needed for attribute query
        if command == self.__class__.ATTRIBUTE_QUERY_CMD:
            len_attribute_names = len(options.attribute_names)
            if len_attribute_names > 0:
                len_attribute_friendly_names = len(
                                            options.attribute_friendly_names)
                if (len_attribute_friendly_names > 0 and
                    len_attribute_friendly_names != len_attribute_names):
                    parser.error("The -f/--attribute_friendly_name option must "
                                 "be set so that it corresponds exactly with "
                                 "the number of -a/--attribute_name options "
                                 "set")
                else:
                    options.attribute_friendly_names = [None
                                                        ]*len_attribute_names
                    
                len_attribute_formats = len(options.attribute_formats)
                
                # A single format setting is allowed which will be applied to 
                # all attributes queried for, else set a format individually for
                # each attribute set
                if len_attribute_formats not in (len_attribute_names, 1):
                    parser.error("The -F/--attribute_format option should be "
                                 "set either once to denote all attributes, or "
                                 "once for each attribute set with the "
                                 "-a/--attribute_name option")
                 
                # If a single format was for more than one attribute, then 
                # expand this so that it is a list of the same size - makes 
                # easier for processing
                if len_attribute_names > 1 and len_attribute_formats == 1:
                    options.attribute_formats = options.attribute_formats * \
                                                    len_attribute_names
  
        missing_vals = []
        for i in (self.__class__.__slots__):
            val = getattr(options, i, None)
            if val == '':
                missing_vals.append(i)
            else:
                setattr(self, i, val)
                
        if len(missing_vals) > 0:
            parser.error('Missing option or options: %r' % missing_vals)
            
        return command
    
    def _set_query_common_attrs(self, query):
        """Set attributes common to both types of SAML query"""
        query.version = SAMLVersion(SAMLVersion.VERSION_20)
        query.id = str(uuid4())
        query.issueInstant = datetime.utcnow()
        
        query.issuer = Issuer()
        query.issuer.format = self.issuer_format
        query.issuer.value = self.issuer
        
        query.subject = Subject()
        query.subject.nameID = NameID()
        query.subject.nameID.format = self.subject_id_format
        query.subject.nameID.value = self.subject_id
 
    def create_authz_decision_query(self):
        """Convenience utility to make an Authorisation decision query"""
        authz_decision_query = AuthzDecisionQuery()

        self._set_query_common_attrs(authz_decision_query)
        
        authz_decision_query.resource = self.resource_id
        
        authz_decision_query.actions.append(Action())
        authz_decision_query.actions[-1].namespace = self.action_namespace
        authz_decision_query.actions[-1].value = self.action
            
        return authz_decision_query

    def create_attribute_query(self):
        attr_query = AttributeQuery()

        self._set_query_common_attrs(attr_query)
        
        for i, name in enumerate(self.attribute_names): 
            attribute = Attribute()
            attribute.name = name
            
            if self.attribute_friendly_names[i] is not None:
                attribute.friendlyName = self.attribute_friendly_names[i]
                
            attribute.nameFormat = self.attribute_formats[i]
            
            attr_query.attributes.append(attribute)
        
        return attr_query
                    
    def dispatch(self, command):
        if command == self.__class__.ATTRIBUTE_QUERY_CMD:
            query = self.create_attribute_query()
            binding = AttributeQuerySslSOAPBinding()
            
        elif command == self.__class__.AUTHZ_DECISION_QUERY_CMD:
            query = self.create_authz_decision_query()
            binding = AuthzDecisionQuerySslSOAPBinding()

        binding.sslCACertDir = self.ca_cert_dir
        binding.sslCertFilePath = self.client_cert_filepath
        binding.sslPriKeyFilePath = self.client_prikey_filepath
        binding.clockSkewTolerance = self.clock_skew_tolerance

        response = binding.send(query, uri=self.service_uri)
        
        return response
    
    @classmethod
    def response_successful(cls, response):
        return response.status.statusCode.value == StatusCode.SUCCESS_URI
    
    def display_result(self, response):
        # Convert back to ElementTree instance read for string output
        saml_response_elem = ResponseElementTree.toXML(response)
        
        if self.pretty_print:
            print(prettyPrint(saml_response_elem))
        else:
            print(ElementTree.tostring(saml_response_elem))

    @classmethod
    def main(cls, argv=sys.argv):
        client = cls()
        command = client.parse_command_line(sys.argv)
        if command is None: # top-level help set
            sys.exit(0)
            
        import logging
        if client.debug:
            log_level = logging.NOTSET
        else:
            log_level = logging.CRITICAL
            
        logging.basicConfig(level=log_level)
        
        try:
            response = client.dispatch(command)
        except Exception, e:
            if client.debug:
                raise
            else:
                raise SystemExit(e)
                
        client.display_result(response)
        
        sys.exit(cls.response_successful(response))
    
        
if __name__ == "__main__":
    SamlSoapCommandLineClient.main()
    
        


"""SAML 2.0 bindings module implements SOAP binding for subject query

NERC DataGrid Project
"""
__author__ = "P J Kershaw"
__date__ = "12/02/10"
__copyright__ = "(C) 2010 Science and Technology Facilities Council"
__license__ = "http://www.apache.org/licenses/LICENSE-2.0"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = '$Id$'
from ndg.saml.saml2.core import SubjectQuery
from ndg.saml.saml2.binding.soap.client import SOAPBindingInvalidResponse
from ndg.saml.saml2.binding.soap.client.requestbase import (
    RequestBaseSOAPBinding,)


class SubjectQueryResponseError(SOAPBindingInvalidResponse):
    """SAML Response error from Subject Query"""
    

class SubjectQuerySOAPBinding(RequestBaseSOAPBinding):
    """SAML Subject Query SOAP Binding
    """ 
    __PRIVATE_ATTR_PREFIX = "__"
    __slots__ = ()
    
    QUERY_TYPE = SubjectQuery
    
    def __init__(self, **kw):
        '''Create SOAP Client for a SAML Subject Query'''       
        super(SubjectQuerySOAPBinding, self).__init__(**kw)





"""SAML 2.0 Utilities module for M2Crypto SSL functionality 

NDG SAML
"""
__author__ = "P J Kershaw"
__date__ = "29/05/15"
__copyright__ = "(C) 2009 Science and Technology Facilities Council"
__license__ = "http://www.apache.org/licenses/LICENSE-2.0"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = '$Id$'
import logging
from OpenSSL import SSL, crypto
from OpenSSL.test.test_ssl import verify_cb

from ndg.httpsclient.ssl_peer_verification import ServerSSLCertVerification

from ndg.saml.utils.ssl_context import SSLContextProxyInterface

log = logging.getLogger(__name__)


class SSLContextProxy(SSLContextProxyInterface):
    SSL_PROTOCOL_METHOD = SSL.TLSv1_METHOD
    SSL_VERIFY_DEPTH = 9
    
    __slots__ = ()

    def __call__(self):
        """Create an M2Crypto SSL Context from this objects properties
        :type depth: int
        :param depth: max. depth of certificate to verify against
        :type kw: dict
        :param kw: OpenSSL.SSL.Context keyword arguments
        :rtype: OpenSSL.SSL.Context
        :return: M2Crypto SSL context object
        """
        ctx = SSL.Context(self.__class__.SSL_PROTOCOL_METHOD)
        
        # Configure context according to this proxy's attributes
        if self.sslCertFilePath and self.sslPriKeyFilePath:
            # Pass client certificate (optionally with chain)
            ctx.use_certificate_chain_file(self.sslCertFilePath)
            
            with open(self.sslPriKeyFilePath, 'r') as prikey_file:
                prikey_content = prikey_file.read()
            prikey_file.close()
            
            prikey = crypto.load_privatekey(crypto.FILETYPE_PEM, 
                                            prikey_content, 
                                            self.sslPriKeyPwd)
            
            ctx.use_privatekey(prikey) 

            log.debug("Set client certificate and key in SSL Context")
        else:
            log.debug("No client certificate or key set in SSL Context")
            
        if self.sslCACertFilePath or self.sslCACertDir:
            # Set CA certificates in order to verify peer
            ctx.load_verify_locations(self.sslCACertFilePath, 
                                      self.sslCACertDir)
            mode = SSL.VERIFY_PEER
            ctx.set_verify_depth(self.__class__.SSL_VERIFY_DEPTH)
            
            verify_cb = lambda connection, peerCert, errorStatus, errorDepth, \
                 preverifyOK: preverifyOK
        else:
            mode = SSL.VERIFY_NONE
            log.warning('No CA certificate files set: mode set to '
                        '"verify_none"!  No verification of the server '
                        'certificate will be enforced')
            
        n_ssl_valid_x509_subj_names = len(self.ssl_valid_x509_subj_names)
        if n_ssl_valid_x509_subj_names > 0 or self.ssl_valid_hostname:
            # Set custom callback in order to verify peer certificate DN 
            # against whitelist
            mode = SSL.VERIFY_PEER
            
            if n_ssl_valid_x509_subj_names == 0:
                cert_dn = None
            else:
                cert_dn = self.ssl_valid_x509_subj_names[0]
                
            # Nb. limit - this verification callback can only validate against
            # a single DN not multiples as allowed by the interface class
            ssl_cert_verification = ServerSSLCertVerification(
                                    hostname=self.ssl_valid_hostname,
                                    certDN=cert_dn)
            
            verify_cb = ssl_cert_verification.get_verify_server_cert_func()
            
            log.debug('Set peer certificate Distinguished Name check set in '
                      'SSL Context')
        else:
            log.warning('No peer certificate Distinguished Name check set in '
                        'SSL Context')
            
        ctx.set_verify(mode, verify_cb)
                   
        return ctx
    
    @SSLContextProxyInterface.ssl_valid_x509_subj_names.setter
    def ssl_valid_x509_subj_names(self, value):
        if isinstance(value, basestring):  
            self._ssl_valid_x509_subj_names = [value]        
        else:
            raise TypeError('Expecting a SINGLE string type for "%s" '
                            'attribute; got %r' %
                (SSLContextProxyInterface.SSL_VALID_X509_SUBJ_NAMES_OPTNAME, 
                 type(value)))



"""SAML 2.0 core module

Implementation of SAML 2.0 for NDG Security

NERC DataGrid Project

This implementation is adapted from the Java OpenSAML implementation.  The 
copyright and licence information are included here:

Copyright [2005] [University Corporation for Advanced Internet Development, Inc.]

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
__author__ = "P J Kershaw"
__date__ = "11/08/09"
__copyright__ = "(C) 2009 Science and Technology Facilities Council"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__license__ = "http://www.apache.org/licenses/LICENSE-2.0"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id$"
from datetime import datetime
import logging
import unittest
import uuid

from ndg.saml import importElementTree
ET = importElementTree()

from ndg.saml.saml2.core import (SAMLVersion, Issuer)

try:
    from ndg.saml.saml2.xacml_profile import XACMLAuthzDecisionQuery
    from ndg.saml.xml.etree_xacml_profile import \
        XACMLAuthzDecisionQueryElementTree
    _xacml_support = True
    
except ImportError, e:
    from warnings import warn
    warn('Error importing XACML packages - skipping XACML profile unit ' + \
         'tests module.  (Error is: %s)' % e)
    _xacml_support = False
    
    
if _xacml_support:

    from ndg.xacml.core.context.action import Action
    from ndg.xacml.core.context.environment import Environment
    from ndg.xacml.core.context.request import Request
    from ndg.xacml.core.context.resource import Resource
    from ndg.xacml.core.context.subject import Subject
    
    logging.basicConfig(level=logging.DEBUG)
    
    class XacmlProfileTestCase(unittest.TestCase):
        """Test XML handling for XACML profile elements"""
        ISSUER_DN = "/O=NDG/OU=BADC/CN=attributeauthority.badc.rl.ac.uk"
    
        def _getSingleElementText(self, contextElem, path):
            elems = contextElem.findall(path)
            self.assertEquals(len(elems), 1, "Single element not selected")
            return elems[0].text
    
        def test01(self):
            # Construct a ResourceContent element.
            rcContentsStr = '''<wps:GetCapabilities
        xmlns:ows="http://www.opengis.net/ows/1.1"
        xmlns:wps="http://www.opengis.net/wps/1.0.0"
        xmlns:xlink="http://www.w3.org/1999/xlink"
        xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
        xsi:schemaLocation="http://schemas.opengis.net/wps/1.0.0/wpsGetCapabilities_request.xsd"
        language="en-CA" service="WPS">
        <wps:AcceptVersions>
            <ows:Version>1.0.0</ows:Version>
        </wps:AcceptVersions>
    </wps:GetCapabilities>
    '''
            # Construct XACMLAuthzDecisionQuery including resource content XML.
            rcContentsElem = ET.XML(rcContentsStr)
            resourceContent = ET.Element(
                "{urn:oasis:names:tc:xacml:2.0:context:schema:os}ResourceContent")
            resourceContent.append(rcContentsElem)
            resourceContent.set('TestAttribute', 'Test Value')
    
            resource = Resource()
            resource.resourceContent = resourceContent
    
            request = Request()
            request.subjects.append(Subject())
            request.resources.append(resource)
            request.action = Action()
            request.environment = Environment()
    
            query = XACMLAuthzDecisionQuery()
            query.xacmlContextRequest = request
    
            query.version = SAMLVersion(SAMLVersion.VERSION_20)
            query.id = str(uuid.uuid4())
            query.issueInstant = datetime.utcnow()
            
            query.issuer = Issuer()
            query.issuer.format = Issuer.X509_SUBJECT
            query.issuer.value = self.ISSUER_DN
    
            # Convert to element tree.
            queryElem = XACMLAuthzDecisionQueryElementTree.toXML(query)
            print ET.tostring(queryElem)
    
            # Check some values from query and resource content XML.
            self.assertEqual(queryElem.get("Version"), "2.0")
    
            self.assertEqual(self._getSingleElementText(queryElem,
                            "{urn:oasis:names:tc:SAML:2.0:assertion}Issuer"),
                            self.ISSUER_DN)
    
            self.assertEqual(self._getSingleElementText(queryElem,
                "{urn:oasis:names:tc:xacml:2.0:context:schema:os}Request/"
                "{urn:oasis:names:tc:xacml:2.0:context:schema:os}Resource/"
                "{urn:oasis:names:tc:xacml:2.0:context:schema:os}ResourceContent/"
                "{http://www.opengis.net/wps/1.0.0}GetCapabilities/"
                "{http://www.opengis.net/wps/1.0.0}AcceptVersions/"
                "{http://www.opengis.net/ows/1.1}Version"), "1.0.0")
    
    
        def test02(self):
            # Construct a ResourceContent element.
            rcContentsStr = '''<wps:GetCapabilities
        xmlns:ows="http://www.opengis.net/ows/1.1"
        xmlns:wps="http://www.opengis.net/wps/1.0.0"
        xmlns:xlink="http://www.w3.org/1999/xlink"
        xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
        xsi:schemaLocation="http://schemas.opengis.net/wps/1.0.0/wpsGetCapabilities_request.xsd"
        language="en-CA" service="WPS">
        <wps:AcceptVersions>
            <ows:Version>1.0.0</ows:Version>
        </wps:AcceptVersions>
    </wps:GetCapabilities>
    '''
            # Construct XACMLAuthzDecisionQuery including resource content XML.
            rcContentsElem = ET.XML(rcContentsStr)
            resourceContent = ET.Element(
            "{urn:oasis:names:tc:xacml:2.0:context:schema:os}ResourceContent")
            resourceContent.append(rcContentsElem)
            resourceContent.set('TestAttribute', 'Test Value')
    
            resource = Resource()
            resource.resourceContent = resourceContent
    
            request = Request()
            request.subjects.append(Subject())
            request.resources.append(resource)
            request.action = Action()
            request.environment = Environment()
    
            query = XACMLAuthzDecisionQuery()
            query.xacmlContextRequest = request
    
            query.version = SAMLVersion(SAMLVersion.VERSION_20)
            query.id = str(uuid.uuid4())
            query.issueInstant = datetime.utcnow()
            
            query.issuer = Issuer()
            query.issuer.format = Issuer.X509_SUBJECT
            query.issuer.value = self.ISSUER_DN
    
            # Convert to element tree.
            queryElem = XACMLAuthzDecisionQueryElementTree.toXML(query)
    
            # Convert back to object tree.
            query2 = XACMLAuthzDecisionQueryElementTree.fromXML(queryElem)
    
            # Check some values from the query and the resource content XML.
            self.assertEqual(query2.version, SAMLVersion(SAMLVersion.VERSION_20))
            self.assertEqual(query2.issuer.value, self.ISSUER_DN)
            self.assertEqual(len(query2.xacmlContextRequest.resources), 1)
    
            rcContentsElem2 = query2.xacmlContextRequest.resources[0
                                                            ].resourceContent
            self.assertEqual(self._getSingleElementText(rcContentsElem2,
                "{http://www.opengis.net/wps/1.0.0}GetCapabilities/"
                "{http://www.opengis.net/wps/1.0.0}AcceptVersions/"
                "{http://www.opengis.net/ows/1.1}Version"), "1.0.0")


if __name__ == "__main__":
    unittest.main()



"""Implementation of SAML 2.0 for NDG Security

NERC DataGrid Project

This implementation is adapted from the Java OpenSAML implementation.  The 
copyright and licence information are included here:

Copyright [2005] [University Corporation for Advanced Internet Development, Inc.]

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
__author__ = "P J Kershaw"
__date__ = "22/07/08"
__copyright__ = "(C) 2009 Science and Technology Facilities Council"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__license__ = "http://www.apache.org/licenses/LICENSE-2.0"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id$"

class Config(object):
    """Configuration options
    @type use_lxml: bool
    @cvar use_lxml: Controls whether lxml.etree should be imported instead of
    etree. lxml is required for XPath expressions with conditions.
    """
    use_lxml = None

def importElementTreeAndCElementTree():
    """Imports ElementTree and cElementTree, or the lxml ElementTree API,
    depending on the Config.use_lxml value and whether the lxml package is
    found.
    @rtype: tuple (module, module)
    @return: the ElementTree and cElementTree modules that have been imported -
    None is returned instead of cElementTree if lxml is used.
    """
    cElementTree = None
    if Config.use_lxml is not None:
        if Config.use_lxml:
            from lxml import etree as ElementTree
        else:
            try: # python 2.5
                # Force absolute import to prevent clash with
                # ndg.saml.xml.etree.
                p = __import__('xml.etree', globals(), locals(),
                               ['cElementTree', 'ElementTree'], 0)
                cElementTree = p.cElementTree
                ElementTree = p.ElementTree
            except ImportError:
                # if you've installed it yourself it comes this way
                import cElementTree, ElementTree
    else:
        Config.use_lxml = False
        try:
            from lxml import etree as ElementTree
            Config.use_lxml = True
        except ImportError:
            try: # python 2.5
                p = __import__('xml.etree', globals(), locals(),
                               ['cElementTree', 'ElementTree'], 0)
                cElementTree = p.cElementTree
                ElementTree = p.ElementTree
            except ImportError:
                # if you've installed it yourself it comes this way
                import cElementTree, ElementTree
    return (ElementTree, cElementTree)

def importElementTree():
    """Imports ElementTree or the lxml ElementTree API depending on the
    Config.use_lxml value and whether the lxml package is found.
    @rtype: module
    @return: the element tree module that has been imported
    """
    return importElementTreeAndCElementTree()[0]



"""SAML 2.0 Utilities module for M2Crypto SSL functionality 

NDG SAML
"""
__author__ = "P J Kershaw"
__date__ = "29/05/15"
__copyright__ = "(C) 2009 Science and Technology Facilities Council"
__license__ = "http://www.apache.org/licenses/LICENSE-2.0"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = '$Id$'
import os
import re
from abc import ABCMeta, abstractmethod
import logging

log = logging.getLogger(__name__)


class SSLContextProxyInterface(object):
    """Interface class for SSL proxy to allow interchange between PyOpenSSL and
    M2Crypto HTTPS libraries"""
    __metaclass__ = ABCMeta
    
    SSL_CERT_FILEPATH_OPTNAME = "sslCertFilePath"
    SSL_PRIKEY_FILEPATH_OPTNAME = "sslPrikeyFilePath"
    SSL_PRIKEY_PWD_OPTNAME = "sslPrikeyPwd"
    SSL_CACERT_FILEPATH_OPTNAME = "sslCACertFilePath"
    SSL_CACERT_DIRPATH_OPTNAME = "sslCACertDir"
    SSL_VALID_X509_SUBJ_NAMES_OPTNAME = "ssl_valid_x509_subj_names"
    SSL_VALID_HOST_NAME_OPTNAME = "ssl_valid_x509_subj_names"
    
    OPTNAMES = (
        SSL_CERT_FILEPATH_OPTNAME,
        SSL_PRIKEY_FILEPATH_OPTNAME,
        SSL_PRIKEY_PWD_OPTNAME,
        SSL_CACERT_FILEPATH_OPTNAME,
        SSL_CACERT_DIRPATH_OPTNAME,
        SSL_VALID_X509_SUBJ_NAMES_OPTNAME
    )
    
    __slots__ = (
        "_ssl_cert_filepath",
        "_ssl_prikey_filepath",
        "_ssl_prikey_pwd",
        "_ssl_cacert_filepath",
        "_ssl_ca_cert_dir",
        "_ssl_valid_hostname",
        "_ssl_valid_x509_subj_names"
    )
            
    VALID_DNS_PAT = re.compile(',\s*')

    def __init__(self):
        self._ssl_cert_filepath = None
        self._ssl_prikey_filepath = None
        self._ssl_prikey_pwd = None
        self._ssl_cacert_filepath = None
        self._ssl_ca_cert_dir = None
        self._ssl_valid_hostname = None
        self._ssl_valid_x509_subj_names = []
        
    @abstractmethod
    def __call__(self):
        """Create an SSL Context from this objects properties
        :type depth: int
        :param depth: max. depth of certificate to verify against
        :type kw: dict
        :param kw: SSL Context keyword arguments
        :rtype: SSL Context of wrapped class e.g. M2Crpyto.SSL.Context or
        OpenSSL.SSL.Context depending on the SSL library used in the implemented
        class
        @return SSL context object
        """
        
    def copy(self, sslCtxProxy):
        """Copy settings from another context object
        """
        if not isinstance(sslCtxProxy, SSLContextProxyInterface):
            raise TypeError('Expecting %r for copy method input object; '
                            'got %r' % (SSLContextProxyInterface, 
                                        type(sslCtxProxy)))
        
        for name in self.__class__.OPTNAMES:
            setattr(self, name, getattr(sslCtxProxy, name))
            
    @property
    def sslCertFilePath(self):
        return self._ssl_cert_filepath
    
    @sslCertFilePath.setter
    def sslCertFilePath(self, filePath):
        "Set X.509 cert/cert chain file path property method"
        
        if isinstance(filePath, basestring):
            filePath = os.path.expandvars(filePath)
            
        elif filePath is not None:
            raise TypeError("X.509 cert. file path must be a valid string")
        
        self._ssl_cert_filepath = filePath
        
    @property
    def sslCACertFilePath(self):
        """Get file path for list of CA cert or certs used to validate SSL 
        connections
        
        :rtype sslCACertFilePath: basestring
        :return sslCACertFilePathList: file path to file containing concatenated
        PEM encoded CA certificates."""
        return self._ssl_cacert_filepath
    
    @sslCACertFilePath.setter
    def sslCACertFilePath(self, value):
        """Set CA cert file path
        
        :type sslCACertFilePath: basestring, list, tuple or None
        :param sslCACertFilePath: file path to CA certificate file.  If None
        then the input is quietly ignored."""
        if isinstance(value, basestring):
            self._ssl_cacert_filepath = os.path.expandvars(value)
            
        elif value is None:
            self._ssl_cacert_filepath = value
            
        else:
            raise TypeError("Input CA Certificate file path must be "
                            "a valid string or None type: %r" % type(value)) 
       
    @property
    def sslCACertDir(self):
        """Get file path for list of CA cert or certs used to validate SSL 
        connections
        
        :rtype sslCACertDir: basestring
        :return sslCACertDirList: directory containing PEM encoded CA 
        certificates."""
        return self._ssl_ca_cert_dir
    
    @sslCACertDir.setter
    def sslCACertDir(self, value):
        """Set CA cert or certs to validate AC signatures, signatures
        of Attribute Authority SOAP responses and SSL connections where 
        AA SOAP service is run over SSL.
        
        :type sslCACertDir: basestring
        :param sslCACertDir: directory containing CA certificate files.
        """
        if isinstance(value, basestring):
            self._ssl_ca_cert_dir = os.path.expandvars(value)
        elif value is None:
            self._ssl_ca_cert_dir = value
        else:
            raise TypeError("Input CA Certificate directroy must be "
                            "a valid string or None type: %r" % type(value))      
      
    @property
    def ssl_valid_hostname(self):
        return self._ssl_valid_hostname
    
    @ssl_valid_hostname.setter
    def ssl_valid_hostname(self, value):
        if isinstance(value, basestring):  
            self._ssl_valid_hostname = value          
        else:
            raise TypeError('Expecting list/tuple or basestring type for "%s" '
                        'attribute; got %r' %
                        (SSLContextProxyInterface.SSL_VALID_HOSTNAME_OPTNAME, 
                         type(value)))
        
    @property
    def ssl_valid_x509_subj_names(self):
        return self._ssl_valid_x509_subj_names

    @ssl_valid_x509_subj_names.setter
    def ssl_valid_x509_subj_names(self, value):
        if isinstance(value, basestring):  
            pat = SSLContextProxyInterface.VALID_DNS_PAT
            self._ssl_valid_x509_subj_names = pat.split(value)
            
        else:
            raise TypeError('Expecting list/tuple or basestring type for "%s" '
                    'attribute; got %r' %
                    (SSLContextProxyInterface.SSL_VALID_X509_SUBJ_NAMES_OPTNAME, 
                     type(value)))

    @property
    def sslPriKeyFilePath(self):
        return self._ssl_prikey_filepath
    
    @sslPriKeyFilePath.setter
    def sslPriKeyFilePath(self, filePath):
        "Set ssl private key file path property method"
        
        if isinstance(filePath, basestring):
            filePath = os.path.expandvars(filePath)

        elif filePath is not None:
            raise TypeError("Private key file path must be a valid "
                            "string or None type")
        
        self._ssl_prikey_filepath = filePath

    @property
    def sslPriKeyPwd(self):
        "Get property method for SSL private key"
        return self._ssl_prikey_pwd
    
    @sslPriKeyPwd.setter
    def sslPriKeyPwd(self, sslPriKeyPwd):
        "Set method for ssl private key file password"
        if not isinstance(sslPriKeyPwd, (type(None), basestring)):
            raise TypeError("Signing private key password must be None "
                            "or a valid string")
        
        self._ssl_prikey_pwd = sslPriKeyPwd

    def __getstate__(self):
        '''Enable pickling for use with beaker.session'''
        _dict = {}
        for attrName in SSLContextProxyInterface.__slots__:
            # Ugly hack to allow for derived classes setting private member
            # variables
            if attrName.startswith('__'):
                attrName = "_SSLContextProxyInterface" + attrName
                
            _dict[attrName] = getattr(self, attrName)
            
        return _dict
        
    def __setstate__(self, attrDict):
        '''Enable pickling for use with beaker.session'''
        for attr, val in attrDict.items():
            setattr(self, attr, val)



"""SOAP common package for NDG SAML.  

Initially for use with SAML SOAP Bindings.  This itself
uses ElementTree.  This SOAP interface provides an ElementTree interface to
support it

NERC DataGrid Project
"""
__author__ = "P J Kershaw"
__date__ = "24/07/09"
__copyright__ = "(C) 2009 Science and Technology Facilities Council"
__license__ = "http://www.apache.org/licenses/LICENSE-2.0"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = '$Id: __init__.py 7130 2010-06-30 13:33:07Z pjkersha $'
import logging
log = logging.getLogger(__name__)


class Config(object):
    """Configuration options
    @type use_lxml: bool
    @cvar use_lxml: Controls whether lxml.etree should be imported instead of
    etree. lxml is required for XPath expressions with conditions.
    """
    use_lxml = None

def importElementTree():
    """Imports ElementTree or the lxml ElementTree API depending on the
    Config.use_lxml value and whether the lxml package is found.
    @rtype: module
    @return: the element tree module that has been imported
    """
    if Config.use_lxml is not None:
        if Config.use_lxml:
            from lxml import etree as ElementTree
        else:
            try: # python 2.5
                from xml.etree import ElementTree
            except ImportError:
                # if you've installed it yourself it comes this way
                import ElementTree
    else:
        Config.use_lxml = False
        try:
            from lxml import etree as ElementTree
            Config.use_lxml = True
        except ImportError:
            try: # python 2.5
                from xml.etree import ElementTree
            except ImportError:
                # if you've installed it yourself it comes this way
                import ElementTree
    return ElementTree

class SOAPObject(object):
    """Base class for SOAP envelope, header and body elements"""
    
    ELEMENT_PREFIX = "soap11"
    SOAP11_NS = "http://schemas.xmlsoap.org/soap/envelope/"
    SOAP12_NS = "http://www.w3.org/2003/05/soap-envelope"
    DEFAULT_NS = SOAP11_NS
    
    __slots__ = ()
    
    def create(self):
        raise NotImplementedError()
    
    def parse(self, source):
        raise NotImplementedError()
    
    def serialize(self):
        raise NotImplementedError()
    
    def prettyPrint(self):
        raise NotImplementedError()
      
    
class SOAPEnvelopeBase(SOAPObject):
    """SOAP Envelope"""
    
    DEFAULT_ELEMENT_LOCAL_NAME = "Envelope"
    DEFAULT_ELEMENT_NS = SOAPObject.DEFAULT_NS
    DEFAULT_ELEMENT_NS_PREFIX = SOAPObject.ELEMENT_PREFIX
    
    soapHeader = property()
    soapBody = property()
    
    __slots__ = ()
    
    
class SOAPHeaderBase(SOAPObject):
    """SOAP Header base class"""
    
    DEFAULT_ELEMENT_LOCAL_NAME = "Header"
    DEFAULT_ELEMENT_NS = SOAPObject.DEFAULT_NS
    DEFAULT_ELEMENT_NS_PREFIX = SOAPObject.ELEMENT_PREFIX    
    __slots__ = ()
       
        
class SOAPBodyBase(SOAPObject):
    """SOAP Body base class"""
    
    DEFAULT_ELEMENT_LOCAL_NAME = "Body"
    DEFAULT_ELEMENT_NS = SOAPObject.DEFAULT_NS
    DEFAULT_ELEMENT_NS_PREFIX = SOAPObject.ELEMENT_PREFIX
    fault = property()
    __slots__ = ()
 
        
class SOAPFaultBase(SOAPObject):
    """SOAP Fault"""
    
    DEFAULT_ELEMENT_LOCAL_NAME = "Fault"
    DEFAULT_ELEMENT_NS = SOAPObject.DEFAULT_NS
    DEFAULT_ELEMENT_NS_PREFIX = SOAPObject.ELEMENT_PREFIX
    
    FAULT_CODE_ELEMENT_LOCAL_NAME = "faultcode"
    FAULT_STRING_ELEMENT_LOCAL_NAME = "faultstring"
    FAULT_ACTOR_ELEMENT_LOCAL_NAME = "faultactor"
    DETAIL_ELEMENT_LOCAL_NAME = "detail"
    
    VERSION_MISMATCH_CODE = "VersionMismatch"
    MUST_UNDERSTAND_FAULT_CODE = "MustUnderstand"
    CLIENT_FAULT_CODE = "Client"
    SERVER_FAULT_CODE = "Server"
    
    FAULT_CODES = (
        VERSION_MISMATCH_CODE, 
        MUST_UNDERSTAND_FAULT_CODE, 
        CLIENT_FAULT_CODE, 
        SERVER_FAULT_CODE
    )
    
    __slots__ = ("__faultCode", "__faultString", "__faultActor", "__detail")
    
    def __init__(self, faultString=None, faultCode=None, faultActor=None, 
                 detail=None):
        """Initialise attributes"""
        super(SOAPFaultBase, self).__init__()
        
        if faultCode is None:
            self.__faultCode = None
        else:
            self.faultCode = faultCode
            
        if faultString is None:
            self.__faultString = None
        else:
            self.faultString = faultString
            
        if faultActor is None:
            self.__faultActor = None
        else:
            self.faultActor = faultActor
        
        if detail is None:
            self.__detail = None
        else:
            self.detail = detail

    def _setFaultCode(self, value):
        if not isinstance(value, basestring):
            raise AttributeError('Expecting string type for "faultCode" '
                                 'attribute; got %r' % type(value))
            
        qnameElems = value.split(':')
        if len(qnameElems) == 0:
            raise AttributeError('Expecting Qualified Name for "faultCode" '
                                 'attribute; got %r' % value)
        
        faultCodeFound = [qnameElems[1].startswith(i) 
                          for i in self.__class__.FAULT_CODES]
        if max(faultCodeFound) == False:
            raise AttributeError('Expecting "faultCode" prefixed with one of '
                                 '%r; got %r' % (self.__class__.FAULT_CODES,
                                                 value))
            
        self.__faultCode = value
        
    def _getFaultCode(self):
        return self.__faultCode

    faultCode = property(_getFaultCode, _setFaultCode, 
                         doc="Fault Code")

    def _setFaultString(self, value):
        if not isinstance(value, basestring):
            raise AttributeError('Expecting string type for "faultString" '
                                 'attribute; got %r' % type(value))
        self.__faultString = value
        
    def _getFaultString(self):
        return self.__faultString

    faultString = property(_getFaultString, _setFaultString, 
                           doc="Fault String")

    def _getFaultActor(self):
        return self.__faultActor

    def _setFaultActor(self, value):
        if not isinstance(value, basestring):
            raise AttributeError('Expecting string type for "faultActor" '
                                 'attribute; got %r' % type(value))
        self.__faultActor = value

    faultActor = property(_getFaultActor, _setFaultActor, 
                          doc="Fault Actor")

    def _getDetail(self):
        return self.__detail

    def _setDetail(self, value):
        """No type checking - detail could be an XML element or serialised 
        string content"""
        self.__detail = value

    detail = property(_getDetail, _setDetail, doc="Fault detail")


class SOAPException(Exception):
    """Base SOAP Exception class"""
        
    
class SOAPFaultException(Exception):
    """Raise an exception which also creates a fault object"""
    SOAP_FAULT_CLASS = SOAPFaultBase
    
    def __init__(self, faultString, faultCode, faultActor=None, detail=None):
        super(SOAPFaultException, self).__init__(faultString)
        self.__fault = self.__class__.SOAP_FAULT_CLASS(faultString, faultCode, 
                                                       faultActor=faultActor, 
                                                       detail=detail)
        
    @property    
    def fault(self):
        """Get SOAP fault object"""
        return self.__fault


#!/usr/bin/env python
"""SOAP module unit test module

NERC DataGrid Project
"""
from ndg.saml.test.binding.soap import paste_installed
__author__ = "P J Kershaw"
__date__ = "24/07/09"
__copyright__ = "(C) 2009 Science and Technology Facilities Council"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__license__ = "http://www.apache.org/licenses/LICENSE-2.0"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id: test_soap.py 7134 2010-06-30 13:49:40Z pjkersha $"
import logging
logging.basicConfig(level=logging.DEBUG)

import unittest
import socket
from cStringIO import StringIO
from os import path
try:
    import paste.fixture
    paste_installed = True
except ImportError:
    paste_installed = False
    
from urllib2 import HTTPHandler, URLError

from ndg.soap import SOAPFaultBase
from ndg.soap.etree import SOAPEnvelope, SOAPFault, SOAPFaultException
from ndg.soap.client import UrlLib2SOAPClient, UrlLib2SOAPRequest
from ndg.soap.test import PasteDeployAppServer


class SOAPBindingMiddleware(object):
    """Simple WSGI interface for SOAP service"""
        
    def __call__(self, environ, start_response):
        requestFile = environ['wsgi.input']
        
        print("Server received request from client:\n\n%s" % 
              requestFile.read())
        
        soapResponse = SOAPEnvelope()
        soapResponse.create()
        
        response = soapResponse.serialize()
        start_response("200 OK",
                       [('Content-length', str(len(response))),
                        ('Content-type', 'text/xml')])
        return [response]
    
    
class SOAPTestCase(unittest.TestCase):
    EG_SOAPFAULT_CODE = "%s:%s" % (SOAPFaultBase.DEFAULT_ELEMENT_NS_PREFIX, 
                                   "MustUnderstand")
    EG_SOAPFAULT_STRING = "Can't process element X set with mustUnderstand"
        
    def test01Envelope(self):
        envelope = SOAPEnvelope()
        envelope.create()
        soap = envelope.serialize()
        
        self.assert_(len(soap) > 0)
        self.assert_("Envelope" in soap)
        self.assert_("Body" in soap)
        self.assert_("Header" in soap)
        
        print(envelope.prettyPrint())
        stream = StringIO()
        stream.write(soap)
        stream.seek(0)
        
        envelope2 = SOAPEnvelope()
        envelope2.parse(stream)
        soap2 = envelope2.serialize()
        self.assert_(soap2 == soap)

    def test02CreateSOAPFaultBase(self):
        
        fault = SOAPFaultBase(self.__class__.EG_SOAPFAULT_STRING, 
                              self.__class__.EG_SOAPFAULT_CODE)
        
        self.assert_(fault.faultCode == self.__class__.EG_SOAPFAULT_CODE)
        self.assert_(fault.faultString == self.__class__.EG_SOAPFAULT_STRING)
     
    def _createSOAPFault(self):
        fault = SOAPFault(self.__class__.EG_SOAPFAULT_STRING, 
                          self.__class__.EG_SOAPFAULT_CODE)
        fault.create()
        return fault   
    
    def test03SerialiseSOAPFault(self):
        # Use ElementTree implementation
        fault = self._createSOAPFault()
        faultStr = fault.serialize()
        print(faultStr)
        self.assert_(self.__class__.EG_SOAPFAULT_STRING in faultStr)

    def test04ParseSOAPFault(self):
        fault = self._createSOAPFault()
        faultStr = fault.serialize()
        stream = StringIO()
        stream.write(faultStr)
        stream.seek(0)
        
        fault2 = SOAPFault()
        fault2.parse(stream)
        self.assert_(fault2.faultCode == fault.faultCode)
        self.assert_(fault2.faultString == fault.faultString)
    
    def test05CreateSOAPFaultException(self):
        try:
            raise SOAPFaultException("bad request", SOAPFault.CLIENT_FAULT_CODE)
        
        except SOAPFaultException, e:
            self.assert_(e.fault.faultString == "bad request")
            self.assert_(SOAPFault.CLIENT_FAULT_CODE in e.fault.faultCode)
            e.fault.create()
            self.assert_("bad request" in e.fault.serialize())
            self.assert_(SOAPFault.CLIENT_FAULT_CODE in e.fault.serialize())
            return
        
        self.fail("Expecting SOAPFaultException raised")
        
    def test06CreateSOAPFaultResponse(self):
        # Create full SOAP Response containing a SOAP Fault
        envelope = SOAPEnvelope()
        envelope.body.fault = self._createSOAPFault()
        envelope.create()
        soap = envelope.serialize()
        
        self.assert_(len(soap) > 0)
        self.assert_("Envelope" in soap)
        self.assert_("Body" in soap)
        self.assert_("Header" in soap)
        self.assert_("Fault" in soap)
        
        print(envelope.prettyPrint())
        stream = StringIO()
        stream.write(soap)
        stream.seek(0)
        
        envelope2 = SOAPEnvelope()
        envelope2.parse(stream)
        soap2 = envelope2.serialize()
        self.assert_(soap2 == soap)
            
    
class SOAPServiceTestCase(unittest.TestCase):
    SOAP_SERVICE_PORTNUM = 10080
    ENDPOINT = 'http://localhost:%d/soap' % SOAP_SERVICE_PORTNUM
    THIS_DIR = path.abspath(path.dirname(__file__))   
    
    @unittest.skipIf(not paste_installed, 'Need Paste.Deploy to run '
                     'SOAPServiceTestCase')
    
    def __init__(self, *args, **kwargs):
        """Use paste.fixture to test client/server SOAP interface"""
        self.services = []
        self.disableServiceStartup = False

        wsgiApp = SOAPBindingMiddleware()
        self.app = paste.fixture.TestApp(wsgiApp)
         
        super(SOAPServiceTestCase, self).__init__(*args, **kwargs)
             
    def test01SendRequest(self):
        requestEnvelope = SOAPEnvelope()
        requestEnvelope.create()
        request = requestEnvelope.serialize()
        
        response = self.app.post('/my-soap-endpoint', 
                                 params=request, 
                                 status=200)
        print(response.headers)
        print(response.status)
        print(response.body)

    def test02Urllib2Client(self):
        
        # Paster based service is threaded from this call
        self.addService(app=SOAPBindingMiddleware(), 
                        port=self.__class__.SOAP_SERVICE_PORTNUM)
        
        client = UrlLib2SOAPClient()
        
        # ElementTree based envelope class
        client.responseEnvelopeClass = SOAPEnvelope
        
        request = UrlLib2SOAPRequest()
        request.url = self.__class__.ENDPOINT
        request.envelope = SOAPEnvelope()
        request.envelope.create()
        
        client.openerDirector.add_handler(HTTPHandler())
        try:
            response = client.send(request)
        except URLError, e:
            self.fail("soap_server.py must be running for this test")
        
        print("Response from server:\n\n%s" % response.envelope.serialize())
        
    def addService(self, *arg, **kw):
        """Utility for setting up threads to run Paste HTTP based services with
        unit tests
        
        @param arg: tuple contains ini file path setting for the service
        @type arg: tuple
        @param kw: keywords including "port" - port number to run the service 
        from
        @type kw: dict
        """
        if self.disableServiceStartup:
            return
        
        try:
            self.services.append(PasteDeployAppServer(*arg, **kw))
            self.services[-1].startThread()
            
        except socket.error:
            pass

    def __del__(self):
        """Stop any services started with the addService method and clean up
        the CA directory following the trust roots call
        """
        if hasattr(self, 'services'):
            for service in self.services:
                service.terminateThread()


if __name__ == "__main__":
    unittest.main()


"""SAML 2.0 bindings module implements SOAP binding for XACMLAuthzDecisionQuery

NERC DataGrid Project
"""
__author__ = "R B Wilkinson"
__date__ = "23/12/11"
__copyright__ = "(C) 2011 Science and Technology Facilities Council"
__license__ = "http://www.apache.org/licenses/LICENSE-2.0"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = '$Id$'
from urlparse import urlparse
import logging
log = logging.getLogger(__name__)

from ndg.saml.saml2.binding.soap.client.requestbase import \
                                                        RequestBaseSOAPBinding
from ndg.saml.saml2.xacml_profile import XACMLAuthzDecisionQuery

try:
    from ndg.httpsclient.https import HTTPSContextHandler as HTTPSHandler_

except ImportError:
    from M2Crypto.m2urllib2 import HTTPSHandler as HTTPSHandler_

# Prevent whole module breaking if this is not available - it's only needed for
# XACMLAuthzDecisionQuerySslSOAPBinding
try:
    from ndg.saml.utils.pyopenssl import SSLContextProxy as SSLContextProxy_
    _sslContextProxySupport = True
    
except ImportError:
    try:
        from ndg.saml.utils.m2crypto import SSLContextProxy as SSLContextProxy_
        _sslContextProxySupport = True
        
    except ImportError:
        _sslContextProxySupport = False


class XACMLAuthzDecisionQuerySOAPBinding(RequestBaseSOAPBinding):
    """XACML-SAML Attribute Query SOAP Binding
    
    Nb. Assumes X.509 subject type for query issuer
    """
    SERIALISE_KW = 'serialise'
    DESERIALISE_KW = 'deserialise'
    QUERY_TYPE = XACMLAuthzDecisionQuery
    __slots__ = ()

    def __init__(self, **kw):
        '''Create SOAP Client for SAML Authorization Decision Query'''
        cls = XACMLAuthzDecisionQuerySOAPBinding

        # Default to ElementTree based serialisation/deserialisation
        if cls.SERIALISE_KW not in kw:
            from ndg.saml.xml.etree_xacml_profile \
                import XACMLAuthzDecisionQueryElementTree
            kw[cls.SERIALISE_KW] = XACMLAuthzDecisionQueryElementTree.toXML

        if cls.DESERIALISE_KW not in kw:
            from ndg.saml.xml.etree import ResponseElementTree
            kw[cls.DESERIALISE_KW] = ResponseElementTree.fromXML

        super(XACMLAuthzDecisionQuerySOAPBinding, self).__init__(**kw)


# Copied from AuthzDecisionQuerySslSOAPBinding
class XACMLAuthzDecisionQuerySslSOAPBinding(XACMLAuthzDecisionQuerySOAPBinding):
    """Specialisation of AuthzDecisionQuerySOAPbinding taking in the setting of
    SSL parameters for mutual authentication
    """
    SSL_CONTEXT_PROXY_SUPPORT = _sslContextProxySupport
    __slots__ = ('__sslCtxProxy',)
    
    def __init__(self, **kw):
        if not XACMLAuthzDecisionQuerySslSOAPBinding.SSL_CONTEXT_PROXY_SUPPORT:
            raise ImportError("ndg.security.common.utils.m2crypto import "
                              "failed - missing M2Crypto package?")
        
        # Miss out default HTTPSHandler and set in send() instead
        if 'handlers' in kw:
            raise TypeError("__init__() got an unexpected keyword argument "
                            "'handlers'")
            
        super(XACMLAuthzDecisionQuerySslSOAPBinding, self).__init__(handlers=(), 
                                                                    **kw)
        self.__sslCtxProxy = SSLContextProxy_()

    def send(self, query, **kw):
        """Override base class implementation to pass explicit SSL Context
        """
        if 'uri' in kw:
            parsed_url = urlparse(kw['uri'])
            self.sslCtxProxy.ssl_valid_hostname = parsed_url.netloc.split(':'
                                                                          )[0]
                                                                          
        httpsHandler = HTTPSHandler_(ssl_context=self.sslCtxProxy())
        self.client.openerDirector.add_handler(httpsHandler)
        return super(XACMLAuthzDecisionQuerySslSOAPBinding, self).send(query, 
                                                                       **kw)
        
    @property
    def sslCtxProxy(self):
        """SSL Context Proxy object used for setting up an SSL Context for
        queries
        """
        return self.__sslCtxProxy
            
    def __setattr__(self, name, value):
        """Enable setting of SSLContextProxy attributes as if they were 
        attributes of this class.  This is intended as a convenience for 
        making settings parameters read from a config file
        """
        try:
            super(XACMLAuthzDecisionQuerySslSOAPBinding, self).__setattr__(name, 
                                                                        value)
            
        except AttributeError, e:
            # Coerce into setting SSL Context Proxy attributes
            try:
                setattr(self.sslCtxProxy, name, value)
            except:
                raise e



#!/usr/bin/env python
"""SAML 2.0 Package

NERC DataGrid Project

This implementation is adapted from the Java OpenSAML implementation.  The 
copyright and licence information are included here:

Copyright [2005] [University Corporation for Advanced Internet Development, 
Inc.]

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
__author__ = "P J Kershaw"
__date__ = "10/08/09"
__copyright__ = "(C) 2015 Science and Technology Facilities Council"
__license__ = "http://www.apache.org/licenses/LICENSE-2.0"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = '$Id$'

# Bootstrap setuptools if necessary.
try:
    from setuptools import setup, find_packages
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup, find_packages
   
_longDescription = """\
SAML 2.0 implementation for use with the Earth System Grid Federation Attribute 
and Authorisation Query interfaces.  The implementation is based on the Java 
OpenSAML libraries.  An implementation is provided with ElementTree but it can 
easily be extended to use other Python XML parsers.

Releases
========
        
0.8.1
-----
 * Fixes for SAML response processing - allow for more liberal check of response type
   and for case for status message value not returned
 
0.8.0
-----
 * Re-factored to use ndg-httpsclient for client HTTP calls in place of M2Crypto.
 * decoupled SAML bindings classes from types.

0.7.0
----- 
 * add command line script for making attribute and authorisation decision
   query client calls.

0.6.0
-----
 * added support for SAML 2.0 profile of XACML v2.0 
   (http://docs.oasis-open.org/xacml/2.0/access_control-xacml-2.0-saml-profile-spec-os.pdf),
   specifically the SAML request extensions: ``XACMLAuthzDecisionQuery`` and 
   ``XACMLAuthzDecisionStatement``.  This an alternative to the SAML defined
   ``AuthzDecisionQuery``.  It enables a richer functionality for expressing
   queries and authorisation decisions taking advantage of the full
   capabilities of a XACML PDP.
 * fixed bug in SAML SOAP binding code: ``RequestBaseSOAPBinding`` and derived 
   classes to act as a query factory, instead of container, for thread 
   safety.
                
   Thanks to Richard Wilkinson for these contributions.
                
0.5.5 
-----
 * allow passing a client certificate chain in client HTTPS requests
        
0.5.4
-----
 * fix for ``ndg.saml.saml2.binding.soap.server.wsgi.queryinterface.SOAPQueryInterfaceMiddleware``:
   bug in issuerFormat property setter - setting ``issuerName`` value.
        
0.5.3
-----
 * fix for ``ndg.soap.utils.etree.prettyPrint`` for undeclared Nss.
        
0.5.2
-----
 * fix for applying clock skew property in ``queryinterface`` WSGI middleware,
   and various minor fixes for ``classfactory`` module and ``m2crytpo`` utilities.
        
0.5.1
-----
 * fix for date time parsing where no seconds fraction is present, fixed
   error message for ``InResponseTo`` ID check for Subject Query.
        
0.5
---
 * adds WSGI middleware and clients for SAML SOAP binding and assertion
   query/request profile.
        
It is not a complete implementation of SAML 2.0.  Only those components required
for the NERC DataGrid have been provided (Attribute and AuthZ Decision Query/
Response).  Where possible, stubs have been provided for other classes.
"""

setup(
    name =           		'ndg_saml',
    version =        		'0.8.1',
    description =    		('SAML 2.0 implementation for the NERC DataGrid '
                                'based on the Java OpenSAML library'),
    long_description =		_longDescription,
    author =         		'Philip Kershaw',
    author_email =   		'Philip.Kershaw@stfc.ac.uk',
    maintainer =         	'Philip Kershaw',
    maintainer_email =   	'Philip.Kershaw@stfc.ac.uk',
    url =            		'https://github.com/cedadev/ndg_saml',
    license =                   'http://www.apache.org/licenses/LICENSE-2.0',
    packages =			find_packages(),
    namespace_packages =	['ndg'],
    extras_require = {
        # These additional packages are needed if you wish to use the SOAP 
        # binding, Nb. M2Crypto can be used in place of ndg-httpsclient if
        # required. ndg-httpsclient provides a urllib2 interface to PyOpenSSL
        'soap_binding':  ["ndg-httpsclient", "Paste", "PasteDeploy", 
                          "PasteScript"],
        # Required for the SAML profile to XACML - enables richer functionality
        # for expressing authorisation queries and decisions.
        'xacml_profile': ['ndg_xacml'],
    },
    entry_points={
    'console_scripts': [
        'ndg_saml_client = ndg.saml.utils.command_line_client:SamlSoapCommandLineClient.main',
        ],
    },
    include_package_data=True,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Environment :: Web Environment',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python',
        'Topic :: Security',
        'Topic :: Internet',
        'Topic :: Scientific/Engineering',
        'Topic :: System :: Distributed Computing',
        'Topic :: System :: Systems Administration :: Authentication/Directory',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    zip_safe=False
)



"""ElementTree module for ElementTree representation of objects in XACML 2.0
profile for  SAML 2.0

NERC DataGrid Project
"""
__author__ = "R B Wilkinson"
__date__ = "23/12/11"
__copyright__ = "(C) 2011 Science and Technology Facilities Council"
__license__ = "http://www.apache.org/licenses/LICENSE-2.0"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = '$Id$'

import logging
log = logging.getLogger(__name__)

from ndg.saml import importElementTree
ElementTree = importElementTree()

from ndg.saml.common import SAMLVersion
from ndg.saml.saml2.core import Issuer
from ndg.saml.saml2.xacml_profile import (XACMLAuthzDecisionQuery,
                                          XACMLAuthzDecisionStatement)
from ndg.saml.utils import SAMLDateTime
from ndg.saml.xml import XMLTypeParseError, UnknownAttrProfile
import ndg.saml.xml.etree as etree
from ndg.saml.xml.etree import (IssuerElementTree, QName,
                                setElementTreeImplementationForQName)

from ndg.xacml.core.context.request import Request
from ndg.xacml.core.context.response import Response
from ndg.xacml.parsers.etree.context import (RequestElementTree,
                                             ResponseElementTree)


class XACMLAuthzDecisionQueryElementTree(XACMLAuthzDecisionQuery):
    """Represent a SAML Attribute Query in XML using ElementTree"""

    @classmethod
    def toXML(cls, xacmlAuthzDecisionQuery):
        """Create an XML representation of the input SAML Authorization
        Decision Query object

        @type xacmlAuthzDecisionQuery: saml.saml2.core.AuthzDecisionQuery
        @param xacmlAuthzDecisionQuery: SAML Authorization Decision Query
        @rtype: ElementTree.Element
        @return: Attribute Query as ElementTree XML element
        """
        if not isinstance(xacmlAuthzDecisionQuery, XACMLAuthzDecisionQuery):
            raise TypeError("Expecting %r class got %r" % (XACMLAuthzDecisionQuery, 
                                                    type(xacmlAuthzDecisionQuery)))

        if not xacmlAuthzDecisionQuery.xacmlContextRequest:
            raise AttributeError("No xacmlContextRequest has been set for the "
                                 "XACMLAuthzDecisionQuery")

        issueInstant = SAMLDateTime.toString(xacmlAuthzDecisionQuery.issueInstant)
        attrib = {
            cls.ID_ATTRIB_NAME: xacmlAuthzDecisionQuery.id,
            cls.ISSUE_INSTANT_ATTRIB_NAME: issueInstant,

            # Nb. Version is a SAMLVersion instance and requires explicit cast
            cls.VERSION_ATTRIB_NAME: str(xacmlAuthzDecisionQuery.version),
        }

        tag = str(QName.fromGeneric(cls.DEFAULT_ELEMENT_NAME))
        elem = etree.makeEtreeElement(tag, cls.DEFAULT_ELEMENT_NAME.prefix,
                                      cls.DEFAULT_ELEMENT_NAME.namespaceURI,
                                      **attrib)

        issuerElem = IssuerElementTree.toXML(xacmlAuthzDecisionQuery.issuer)
        elem.append(issuerElem)

        requestElem = RequestElementTree.toXML(
                                    xacmlAuthzDecisionQuery.xacmlContextRequest)
        elem.append(requestElem)

        return elem

    @classmethod
    def fromXML(cls, elem):
        """Parse ElementTree element into a SAML XACMLAuthzDecisionQuery object
        
        @type elem: ElementTree.Element
        @param elem: XML element containing the AuthzDecisionQuery
        @rtype: saml.saml2.core.AuthzDecisionQuery
        @return: AuthzDecisionQuery object
        """
        if not ElementTree.iselement(elem):
            raise TypeError("Expecting %r input type for parsing; got %r" %
                            (ElementTree.Element, elem))

        if QName.getLocalPart(elem.tag) != cls.DEFAULT_ELEMENT_LOCAL_NAME:
            raise XMLTypeParseError("No \"%s\" element found" %
                                    cls.DEFAULT_ELEMENT_LOCAL_NAME)
        
        # Unpack attributes from top-level element
        attributeValues = []
        for attributeName in (cls.VERSION_ATTRIB_NAME,
                              cls.ISSUE_INSTANT_ATTRIB_NAME,
                              cls.ID_ATTRIB_NAME):
            attributeValue = elem.attrib.get(attributeName)
            if attributeValue is None:
                raise XMLTypeParseError('No "%s" attribute found in "%s" '
                                 'element' %
                                 (attributeName,
                                  cls.DEFAULT_ELEMENT_LOCAL_NAME))
                
            attributeValues.append(attributeValue)
        
        authzDecisionQuery = XACMLAuthzDecisionQuery()
        authzDecisionQuery.version = SAMLVersion(attributeValues[0])
        if authzDecisionQuery.version != SAMLVersion.VERSION_20:
            raise NotImplementedError("Parsing for %r is implemented for "
                                      "SAML version %s only; version %s is " 
                                      "not supported" % 
                                      (cls,
                                       SAMLVersion(SAMLVersion.VERSION_20),
                                       SAMLVersion(authzDecisionQuery.version)))
            
        authzDecisionQuery.issueInstant = SAMLDateTime.fromString(
                                                            attributeValues[1])
        authzDecisionQuery.id = attributeValues[2]
        
        for childElem in elem:
            localName = QName.getLocalPart(childElem.tag)
            if localName == Issuer.DEFAULT_ELEMENT_LOCAL_NAME:
                # Parse Issuer
                authzDecisionQuery.issuer = IssuerElementTree.fromXML(childElem)

            elif localName == Request.ELEMENT_LOCAL_NAME:
                # Create XACML context request from Request element.
                authzDecisionQuery.xacmlContextRequest = \
                    RequestElementTree.fromXML(childElem)

            else:
                raise XMLTypeParseError("Unrecognised XACMLAuthzDecisionQuery child "
                                        "element \"%s\"" % localName)
        
        return authzDecisionQuery

class XACMLAuthzDecisionStatementElementTree(XACMLAuthzDecisionStatement):
    @classmethod
    def toXML(cls, xacmlAuthzDecisionStatement):
        if not isinstance(xacmlAuthzDecisionStatement,
                          XACMLAuthzDecisionStatement):
            raise TypeError("Expecting %r class got %r" %
                            (XACMLAuthzDecisionStatement, 
                            type(xacmlAuthzDecisionStatement)))

        if not xacmlAuthzDecisionStatement.xacmlContextResponse:
            raise AttributeError("No xacmlContextResponse has been set for the "
                                 "XACMLAuthzDecisionStatement")

        tag = str(QName.fromGeneric(cls.DEFAULT_ELEMENT_NAME))
        elem = etree.makeEtreeElement(tag, cls.DEFAULT_ELEMENT_NAME.prefix,
                                      cls.DEFAULT_ELEMENT_NAME.namespaceURI)

        xacmlContextResponseElem = ResponseElementTree.toXML(
                            xacmlAuthzDecisionStatement.xacmlContextResponse)
        elem.append(xacmlContextResponseElem)

        if xacmlAuthzDecisionStatement.xacmlContextRequest:
            xacmlContextRequestElem = RequestElementTree.toXML(
                                xacmlAuthzDecisionStatement.xacmlContextRequest)
            elem.append(xacmlContextRequestElem)

        return elem

    @classmethod
    def fromXML(cls, elem):
        if not ElementTree.iselement(elem):
            raise TypeError("Expecting %r input type for parsing; got %r" %
                            (ElementTree.Element, elem))

        if QName.getLocalPart(elem.tag) != cls.DEFAULT_ELEMENT_LOCAL_NAME:
            raise XMLTypeParseError("No \"%s\" element found" %
                                    cls.DEFAULT_ELEMENT_LOCAL_NAME)

        authzDecisionStatement = XACMLAuthzDecisionStatement()

        for childElem in elem:
            localName = QName.getLocalPart(childElem.tag)
            if localName == Response.ELEMENT_LOCAL_NAME:
                # Create XACML context request from Response element.
                authzDecisionStatement.xacmlContextResponse = \
                    ResponseElementTree.fromXML(childElem)

            elif localName == Request.ELEMENT_LOCAL_NAME:
                # Create XACML context request from Request element.
                authzDecisionStatement.xacmlContextRequest = \
                    RequestElementTree.fromXML(childElem)

            else:
                raise XMLTypeParseError("Unrecognised XACMLAuthzDecisionQuery child "
                                        "element \"%s\"" % localName)

        return authzDecisionStatement

def setElementTreeMap():
    """
    Sets a mapping of XACMLAuthzDecisionStatement element name to the
    corresponding ElementTree class so that statements of this type
    can be processed.
    """
    setElementTreeImplementationForQName(
                XACMLAuthzDecisionStatement.DEFAULT_ELEMENT_NAME,
                XACMLAuthzDecisionStatementElementTree)



"""XACML 2.0 profile for SAML 2.0 module

NERC DataGrid Project
"""
__author__ = "R B Wilkinson"
__date__ = "23/12/11"
__copyright__ = "(C) 2011 Science and Technology Facilities Council"
__license__ = "http://www.apache.org/licenses/LICENSE-2.0"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = '$Id$'

from ndg.saml.common.xml import SAMLConstants, QName
from ndg.saml.saml2.core import RequestAbstractType, Statement

from ndg.xacml.core.context.request import Request
from ndg.xacml.core.context.response import Response


class XACMLAuthzDecisionQuery(RequestAbstractType):
    '''SAML 2.0 XACML Profile XACMLAuthzDecisionQuery
    
    :cvar DEFAULT_ELEMENT_LOCAL_NAME: Element local name.
    :type DEFAULT_ELEMENT_LOCAL_NAME: string
    :cvar DEFAULT_ELEMENT_NAME: Default element name.
    :type DEFAULT_ELEMENT_NAME: string
    :cvar TYPE_LOCAL_NAME: Local name of the XSI type.
    :type TYPE_LOCAL_NAME: string
    :cvar TYPE_NAME: QName of the XSI type.
    :type TYPE_NAME: string
    :cvar RETURN_CONTEXT_ATTRIB_NAME: ReturnContext attribute name.
    :type RETURN_CONTEXT_ATTRIB_NAME: string
   
    :ivar inputContextOnly: InputContextOnly attribute value.
    :type inputContextOnly: bool
    :ivar returnContext: ReturnContext attribute value.
    :type returnContext: bool
    :ivar xacmlContextRequest: XACML context request
    :type xacmlContextRequest: ndg.xacml.core.context.request.Request
    '''

    # Element local name.
    DEFAULT_ELEMENT_LOCAL_NAME = "XACMLAuthzDecisionQuery"

    # Default element name.
    DEFAULT_ELEMENT_NAME = QName(SAMLConstants.SAML2_XACML_PROTOCOL_NS, 
                                 DEFAULT_ELEMENT_LOCAL_NAME,
                                 SAMLConstants.SAML2_XACML_PROTOCOL_PREFIX)

    # Local name of the XSI type.
    TYPE_LOCAL_NAME = "XACMLAuthzDecisionQueryType"

    # QName of the XSI type.
    TYPE_NAME = QName(SAMLConstants.SAML2_XACML_PROTOCOL_NS, 
                      TYPE_LOCAL_NAME,
                      SAMLConstants.SAML2_XACML_PROTOCOL_PREFIX)

    # InputContextOnly attribute name
    INPUT_CONTEXT_ONLY_ATTRIB_NAME = "InputContextOnly"

    # ReturnContext attribute name
    RETURN_CONTEXT_ATTRIB_NAME = "ReturnContext"

    __slots__ = (
       '__inputContextOnly',
       '__returnContext',
       '__xacmlContextRequest'
    )
    
    def __init__(self):
        '''Create new authorisation decision query
        '''
        super(XACMLAuthzDecisionQuery, self).__init__()

        # Input context only attribute value
        self.__inputContextOnly = None
    
        # Return context attribute value
        self.__returnContext = None

        # XACML request context child element
        self.__xacmlContextRequest = None

    def __getstate__(self):
        '''Enable pickling
        
        :return: object's attribute dictionary
        :rtype: dict
        '''
        _dict = super(XACMLAuthzDecisionQuery, self).__getstate__()
        for attrName in XACMLAuthzDecisionQuery.__slots__:
            # Ugly hack to allow for derived classes setting private member
            # variables
            if attrName.startswith('__'):
                attrName = "_XACMLAuthzDecisionQuery" + attrName
                
            _dict[attrName] = getattr(self, attrName)
            
        return _dict

    def _getInputContextOnly(self):
        '''Get the InputContextOnly attribute value of this query

        :return: InputContextOnly value
        :rtype: bool
        '''
        return self.__inputContextOnly
    
    def _setInputContextOnly(self, value):
        '''Sets the InputContextOnly attribute value of this query.
        
        :param value: the new InputContextOnly attribute value
        :type value: bool
        :raise TypeError: if incorrect input type 
        '''
        if not isinstance(value, basestring):
            raise TypeError('Expecting string type for "InputContextOnly" '
                            'attribute; got %r instead' % type(value))
        
        self.__inputContextOnly = value
    
    inputContextOnly = property(fget=_getInputContextOnly,
                                fset=_setInputContextOnly,
                                doc="Determines whether the decision is made "
                                "using information in the decision query only "
                                "- unused")

    def _getReturnContext(self):
        '''Get the ReturnContext attribute value of this query

        :return: ReturnContext value
        :rtype: bool
        '''
        return self.__returnContext
    
    def _setReturnContext(self, value):
        '''Sets the ReturnContext attribute value of this query.
        
        :param value: the new ReturnContext attribute value
        :type value: bool
        :raise TypeError: if incorrect input type 
        '''
        if not isinstance(value, basestring):
            raise TypeError('Expecting string type for "ReturnContext" '
                            'attribute; got %r instead' % type(value))
        
        self.__returnContext = value
    
    returnContext = property(fget=_getReturnContext,
                             fset=_setReturnContext,
                            doc="Determines whether the request context is"
                            "response")

    def _getXacmlContextRequest(self):
        '''
        Gets the XACML context request.

        :rtype: 
        :return: XACML context request
        '''
        return self.__xacmlContextRequest

    def _setXacmlContextRequest(self, value):
        '''
        Sets the XacmlContextRequest.

        :param value: XacmlContextRequest
        :raise TypeError: input value is incorrect type
        '''
        if not isinstance(value, Request):
            raise TypeError('Expecting %r type for "decision" attribute; '
                            'got %r instead' % (Request, type(value)))
        self.__xacmlContextRequest = value

    xacmlContextRequest = property(_getXacmlContextRequest,
                                   _setXacmlContextRequest, 
                                   doc="XACML context request")

    def getOrderedChildren(self):
        '''Return attributes for this element as a tuple

        :return: attributes for this element
        :rtype: tuple
        '''
        children = []

        superChildren = super(XACMLAuthzDecisionQuery, self).getOrderedChildren()
        if superChildren:
            children.extend(superChildren)

        children.extend(self.__xacmlContextRequest)

        if len(children) == 0:
            return None

        return tuple(children)


class XACMLAuthzDecisionStatement(Statement):
    '''SAML 2.0 XACML Profile XACMLAuthzDecisionQuery
    
    :cvar DEFAULT_ELEMENT_LOCAL_NAME: Element local name.
    :type DEFAULT_ELEMENT_LOCAL_NAME: string
    :cvar DEFAULT_ELEMENT_NAME: Default element name.
    :type DEFAULT_ELEMENT_NAME: string
    :cvar TYPE_LOCAL_NAME: Local name of the XSI type.
    :type TYPE_LOCAL_NAME: string
    :cvar TYPE_NAME: QName of the XSI type.
    :type TYPE_NAME: string
    '''

    # Element local name.
    DEFAULT_ELEMENT_LOCAL_NAME = "XACMLAuthzDecisionStatement"

    # Default element name.
    DEFAULT_ELEMENT_NAME = QName(SAMLConstants.SAML2_XACML_ASSERTION_NS,
                                 DEFAULT_ELEMENT_LOCAL_NAME,
                                 SAMLConstants.SAML2_XACML_ASSERTION_PREFIX)

    # Local name of the XSI type.
    TYPE_LOCAL_NAME = "XACMLAuthzDecisionStatementType"

    # QName of the XSI type.
    TYPE_NAME = QName(SAMLConstants.SAML2_XACML_ASSERTION_NS, 
                      TYPE_LOCAL_NAME,
                      SAMLConstants.SAML2_XACML_ASSERTION_PREFIX)
    __slots__ = (
        '__xacmlContextRequest',
        '__xacmlContextResponse'
    )
    
    def __init__(self):
        '''Create new authorisation decision statement
        '''
        super(XACMLAuthzDecisionStatement, self).__init__(
                    namespaceURI=SAMLConstants.SAML2_XACML_ASSERTION_NS, 
                    namespacePrefix=SAMLConstants.SAML2_XACML_ASSERTION_PREFIX)
        self.__xacmlContextRequest = None
        self.__xacmlContextResponse = None

    def __getstate__(self):
        '''Enable pickling
        
        :return: object's attribute dictionary
        :rtype: dict
        '''
        _dict = super(XACMLAuthzDecisionStatement, self).__getstate__()
        for attrName in XACMLAuthzDecisionStatement.__slots__:
            # Ugly hack to allow for derived classes setting private member
            # variables
            if attrName.startswith('__'):
                attrName = "_XACMLAuthzDecisionStatement" + attrName
                
            _dict[attrName] = getattr(self, attrName)
            
        return _dict

    def _getXacmlContextRequest(self):
        '''Gets the XACML context Request child element.

        :return: Request value
        :rtype: ndg.xacml.core.context.request.Request
        '''
        return self.__xacmlContextRequest
    
    def _setXacmlContextRequest(self, value):
        '''Sets  the XACML context Request child element.
        
        :param value: the new Request attribute value
        :type value: ndg.xacml.core.context.request.Request
        :raise TypeError: if incorrect input type 
        '''
        if not isinstance(value, Request):
            raise TypeError('Expecting string type for "Request" '
                            'attribute; got %r instead' % type(value))
        
        self.__xacmlContextRequest = value
    
    xacmlContextRequest = property(fget=_getXacmlContextRequest,
                                   fset=_setXacmlContextRequest,
                                   doc="XACML context Request")

    def _getXacmlContextResponse(self):
        '''Gets the XACML context Response child element.

        :return: Response value
        :rtype: ndg.xacml.core.context.request.Response
        '''
        return self.__xacmlContextResponse
    
    def _setXacmlContextResponse(self, value):
        '''Sets  the XACML context Response child element.
        
        :param value: the new Response attribute value
        :type value: ndg.xacml.core.context.request.Response
        :raise TypeError: if incorrect input type 
        '''
        if not isinstance(value, Response):
            raise TypeError('Expecting string type for "Response" '
                            'attribute; got %r instead' % type(value))
        
        self.__xacmlContextResponse = value
    
    xacmlContextResponse = property(fget=_getXacmlContextResponse,
                                    fset=_setXacmlContextResponse,
                                    doc="XACML context Response")
